@@ -1,7 +1,7 @@
 (in-package :maxima)
 
 ;; wxMaxima xml format (based on David Drysdale MathML printing)
-;; Andrej Vodopivec, 2004-2006
+;; Andrej Vodopivec, 2004-2007
 
 ;; MathML-printing
 ;; Created by David Drysdale (DMD), December 2002/January 2003
@@ -15,19 +15,23 @@
 
 ;; Method:
 
-;; Producing wxxml from a macsyma internal expression is done by
+;; Producing wxxml from a maxima internal expression is done by
 ;; a reversal of the parsing process.  Fundamentally, a
 ;; traversal of the expression tree is produced by the program,
 ;; with appropriate substitutions and recognition of the
 ;; infix / prefix / postfix / matchfix relations on symbols. Various
 ;; changes are made to this so that MathML will like the results.
 
-;;(macsyma-module wxxml)
-
 (declare-top
- (special lop rop ccol $gcprint texport $labels $inchar maxima-main-dir)
+ (special lop rop $inchar)
  (*expr wxxml-lbp wxxml-rbp))
 
+($put '$wxmaxima `((mlist simp) 0 8 2) '$version)
+
+(setf (get '$inchar 'assign) 'neverset)
+
+(defvar *var-tag* '("<v>" "</v>"))
+
 (defun wxxml (x l r lop rop)
   ;; x is the expression of interest; l is the list of strings to its
   ;; left, r to its right. lop and rop are the operators on the left
@@ -39,12 +43,22 @@
              (> (wxxml-lbp rop) (wxxml-rbp (caar x))))
          (wxxml-paren x l r))
         ;; special check needed because macsyma notates arrays peculiarly
-        ((memq 'array (cdar x)) (wxxml-array x l r))
+        ((member 'array (cdar x) :test #'eq) (wxxml-array x l r))
         ;; dispatch for object-oriented wxxml-ifiying
         ((get (caar x) 'wxxml) (funcall (get (caar x) 'wxxml) x l r))
         ((equal (get (caar x) 'dimension) 'dimension-infix)
          (wxxml-infix x l r))
-        (t (wxxml-function x l r nil))))
+	((equal (get (caar x) 'dimension) 'dimension-match)
+         (wxxml-matchfix-dim x l r))
+	((equal (get (caar x) 'dimension) 'dimension-nary)
+	 (wxxml-nary x l r))
+        (t (wxxml-function x l r))))
+
+(defmacro make-tag (val tag)
+  ``((wxxmltag simp) ,,val ,,tag))
+
+(defun $wxxmltag (val tag)
+  (make-tag ($sconcat val) ($sconcat tag)))
 
 (defun string-substitute (newstring oldchar x &aux matchpos)
   (setq matchpos (position oldchar x))
@@ -55,25 +69,38 @@
 		   (string-substitute newstring oldchar
 				      (subseq x (1+ matchpos))))))
 
+(defun wxxml-fix-string (x)
+  (if (stringp x)
+      (let* ((tmp-x (string-substitute "&amp;" #\& x))
+	     (tmp-x (string-substitute "&lt;" #\< tmp-x))
+	     (tmp-x (string-substitute "&gt;" #\> tmp-x)))
+	tmp-x)
+      x))
+
 ;;; First we have the functions which are called directly by wxxml and its
 ;;; descendents
 
-(defun wxxml-atom (x l r)
+(defun wxxml-atom (x l r &aux tmp-x)
   (append l
           (list (cond ((numberp x) (wxxmlnumformat x))
                       ((typep x 'structure-object)
-		       (strcat "<v>Lisp structure: " (type-of x) " </v>"))
-                      ((mstringp x)
-                       (let*
-                           ((tmp-x (maybe-invert-string-case (symbol-name (stripdollar x))))
-                            (tmp-x (string-substitute "&amp;" #\& tmp-x))
-                            (tmp-x (string-substitute "&lt;" #\< tmp-x))
-                            (tmp-x (string-substitute "&gt;" #\> tmp-x)))
-                         (strcat "<st>" tmp-x "</st>")))
-                      ((and (symbolp x) (get x 'wxxmlword))
-		       (wxxml-stripdollar (get x 'wxxmlword)))
+		       (format nil "<v>Structure [~A]</v>" (type-of x)))
+		      ((hash-table-p x)
+		       (format nil "<v>HashTable</v>"))
+		      ((stringp x)
+		       (setq tmp-x (wxxml-fix-string x))
+		       (if (and (boundp '$stringdisp) $stringdisp)
+			   (setq tmp-x (format nil "\"~a\"" tmp-x)))
+		       (concatenate 'string "<st>" tmp-x "</st>"))
+		      ((arrayp x)
+		       (format nil "<v>Lisp array [~{~a~^,~}]</v>"
+			       (array-dimensions x)))
+                      ((and (symbolp x) (get x 'wxxmlword)))
                       ((and (symbolp x) (get x 'reversealias))
                        (wxxml-stripdollar (get x 'reversealias)))
+		      ((streamp x)
+		       (format nil "<v>Stream [~A]</v>"
+			       (stream-element-type x)))
                       (t (wxxml-stripdollar x))
 		      ))
 	  r))
@@ -91,36 +118,48 @@
 		  (setq firstpart
 			(nreverse (cdr (member 'e (reverse r)
                                                :test #'string-equal))))
-		  (format nil 
+		  (if (char= (cadr exponent) #\+)
+		      (setq exponent (cddr exponent))
+		      (setq exponent (cdr exponent)))
+		  (format nil
 			  "<r><n>~{~c~}</n><h>*</h><e><n>10</n><n>~{~c~}</n></e></r>"
-			  firstpart (cdr exponent))))))))
+			  firstpart exponent)))))))
 
-(defun wxxml-stripdollar (sym)
+(defun wxxml-stripdollar (sym &aux pname)
   (or (symbolp sym)
-      (return-from wxxml-stripdollar sym))
-  (let* ((pname (maybe-invert-string-case (symbol-name sym)))
-         (pname (if (memq (elt pname 0) '(#\$ #\&)) (subseq pname 1) pname))
-         (pname (string-substitute "&amp;" #\& pname))
-         (pname (string-substitute "&gt;" #\> pname))
-         (pname (string-substitute "&lt;" #\< pname)))
-    (concatenate 'string "<v>" pname "</v>")))
+      (return-from wxxml-stripdollar
+	(wxxml-fix-string (format nil "~a" sym))))
+  (setq pname (maybe-invert-string-case (symbol-name sym)))
+  (setq pname (cond ((and (> (length pname) 0)
+			  (member (elt pname 0) '(#\$ #\&) :test #'eq))
+		     (subseq pname 1))
+		    ((and (> (length pname) 0)
+			  (equal (elt pname 0) #\%))
+		     (if $noundisp
+			 (concatenate 'string "'"
+				      (subseq pname 1))
+			 (subseq pname 1)))
+		    ($lispdisp
+		     (concatenate 'string "?" pname))
+		    (t pname)))
+  (setq pname (wxxml-fix-string pname))
+  (concatenate 'string (car *var-tag*) pname (cadr *var-tag*)))
 
 (defun wxxml-paren (x l r)
   (wxxml x (append l '("<p>")) (cons "</p>" r) 'mparen 'mparen))
 
-(defun wxxml-array (x l r)
-  (let ((f))
-    (if (eq 'mqapply (caar x))
-	(setq f (cadr x)
-	      x (cdr x)
-	      l (wxxml f (append l (list "<i><r>")) nil
-                       'mparen 'mparen))
-	(setq f (caar x)
-	      l (wxxml (wxxmlword f) (append l '("<i><r>"))
-		       nil lop 'mfunction)))
-    (setq r (nconc (wxxml-list (cdr x) (list "</r><r>")
-                               (list "</r></i>") "<v>,</v>") r))
-    (nconc l r)))
+(defun wxxml-array (x l r &aux f)
+  (if (eq 'mqapply (caar x))
+      (setq f (cadr x)
+	    x (cdr x)
+	    l (wxxml f (append l (list "<i><p>")) (list "</p>")
+		     'mparen 'mparen))
+      (setq f (caar x)
+	    l (wxxml f (append l '("<i><r>"))
+		     (list "</r>") lop 'mfunction)))
+  (setq r (nconc (wxxml-list (cdr x) (list "<r>")
+			     (list "</r></i>") "<v>,</v>") r))
+  (nconc l r))
 
 ;; set up a list , separated by symbols (, * ...)  and then tack on the
 ;; ending item (e.g. "]" or perhaps ")"
@@ -136,12 +175,14 @@
 
 ;; we could patch this so sin x rather than sin(x), but instead we made
 ;; sin a prefix operator
-(defun wxxml-function (x l r op) op
-       (setq l (wxxml (wxxmlword (caar x)) (append l '("<fn>"))
-		      nil 'mparen 'mparen)
-	     r (wxxml (cons '(mprogn) (cdr x)) nil (append '("</fn>") r)
-		      'mparen 'mparen))
-       (append l r))
+(defun wxxml-function (x l r)
+  (setq l
+	(let ((*var-tag* '("<fnm>" "</fnm>")))
+	  (wxxml (caar x) (append l '("<fn>"))
+		 nil 'mparen 'mparen))
+	r (wxxml (cons '(mprogn) (cdr x)) nil (append '("</fn>") r)
+		 'mparen 'mparen))
+  (append l r))
 
 ;;; Now we have functions which are called via property lists
 
@@ -159,14 +200,20 @@
 
 (defun wxxml-nary (x l r)
   (let* ((op (caar x))
-         (sym (wxxmlsym op))
+	 (sym (cond ((eq op 'mtimes)
+		     (if $stardisp
+			 "<t>*</t>"
+			 "<h>*</h>"))
+	       ((eq (get op 'dimension) 'dimension-nary)
+		(wxxml-dissym-to-string (get op 'dissym)))
+	       ((wxxmlsym op))))
          (y (cdr x))
          (ext-lop lop)
          (ext-rop rop))
     (cond ((null y)
-	   (wxxml-function x l r t)) ; this should not happen
+	   (wxxml-function x l r)) ; this should not happen
           ((null (cdr y))
-	   (wxxml-function x l r t)) ; this should not happen, too
+	   (wxxml-function x l r)) ; this should not happen, too
           (t (do ((nl) (lop ext-lop op)
                   (rop op (if (null (cdr y)) ext-rop op)))
                  ((null (cdr y))
@@ -185,11 +232,25 @@
 	x (wxxml-list (cdr x) nil r "<t>,</t>"))
   (append l x))
 
+(defun wxxml-matchfix-dim (x l r)
+  (setq l (append l
+		  (list (wxxml-dissym-to-string (car (get (caar x) 'dissym)))))
+	r (append (list (wxxml-dissym-to-string (cdr (get (caar x) 'dissym))))
+		  r)
+	x (wxxml-list (cdr x) nil r "<t>,</t>"))
+  (append l x))
+
+(defun wxxml-dissym-to-string (lst &aux pname)
+  (setq pname
+	(wxxml-fix-string (format nil "~{~a~}" lst)))
+  (concatenate 'string "<v>" pname "</v>"))
+
 (defun wxxmlsym (x)
   (or (get x 'wxxmlsym)
       (get x 'strsym)
-      (get x 'dissym)
-      (stripdollar x)))
+      (and (get x 'dissym)
+	   (list (wxxml-dissym-to-string (get x 'dissym))))
+      (list (stripdollar x))))
 
 (defun wxxmlword (x)
   (or (get x 'wxxmlword)
@@ -201,9 +262,7 @@
 (defun wxxml-bigfloat (x l r)
   (append l '("<n>") (fpformat x) '("</n>") r))
 
-(defprop mprog  "<t>block</t>" wxxmlword)
-(defprop %erf   "<t>erf</t>"   wxxmlword)
-(defprop $erf   "<t>erf</t>"   wxxmlword)
+(defprop mprog  "<fnm>block</fnm>" wxxmlword)
 (defprop $true  "<t>true</t>"  wxxmlword)
 (defprop $false "<t>false</t>" wxxmlword)
 
@@ -243,53 +302,100 @@
 ;; Greek characters
 ;;
 (defprop $%alpha "<g>%alpha</g>" wxxmlword)
+(defprop $alpha "<g>alpha</g>" wxxmlword)
 (defprop $%beta "<g>%beta</g>" wxxmlword)
+(defprop $beta "<g>beta</g>" wxxmlword)
 (defprop $%gamma "<g>%gamma</g>" wxxmlword)
+(defprop %gamma "<g>gamma</g>" wxxmlword)
 (defprop $%delta "<g>%delta</g>" wxxmlword)
+(defprop $delta "<g>delta</g>" wxxmlword)
 (defprop $%epsilon "<g>%epsilon</g>" wxxmlword)
+(defprop $epsilon "<g>epsilon</g>" wxxmlword)
 (defprop $%zeta "<g>%zeta</g>" wxxmlword)
 (defprop $%eta "<g>%eta</g>" wxxmlword)
+(defprop $eta "<g>eta</g>" wxxmlword)
 (defprop $%theta "<g>%theta</g>" wxxmlword)
+(defprop $theta "<g>theta</g>" wxxmlword)
 (defprop $%iota "<g>%iota</g>" wxxmlword)
+(defprop $iota "<g>iota</g>" wxxmlword)
 (defprop $%kappa "<g>%kappa</g>" wxxmlword)
+(defprop $kappa "<g>kappa</g>" wxxmlword)
 (defprop $%lambda "<g>%lambda</g>" wxxmlword)
+(defprop $lambda "<g>lambda</g>" wxxmlword)
 (defprop $%mu "<g>%mu</g>" wxxmlword)
+(defprop $mu "<g>mu</g>" wxxmlword)
 (defprop $%nu "<g>%nu</g>" wxxmlword)
+(defprop $nu "<g>nu</g>" wxxmlword)
 (defprop $%xi "<g>%xi</g>" wxxmlword)
+(defprop $xi "<g>xi</g>" wxxmlword)
 (defprop $%omicron "<g>%omicron</g>" wxxmlword)
-(defprop $%pi "<s>%pi</s>" wxxmlword)
+(defprop $omicron "<g>omicron</g>" wxxmlword)
+(defprop $%pi "<g>%pi</g>" wxxmlword)
+(defprop $pi "<g>pi</g>" wxxmlword)
 (defprop $%rho "<g>%rho</g>" wxxmlword)
+(defprop $rho "<g>rho</g>" wxxmlword)
 (defprop $%sigma "<g>%sigma</g>" wxxmlword)
+(defprop $sigma "<g>sigma</g>" wxxmlword)
 (defprop $%tau "<g>%tau</g>" wxxmlword)
+(defprop $tau "<g>tau</g>" wxxmlword)
 (defprop $%upsilon "<g>%upsilon</g>" wxxmlword)
+(defprop $upsilon "<g>upsilon</g>" wxxmlword)
 (defprop $%phi "<g>%phi</g>" wxxmlword)
+(defprop $phi "<g>phi</g>" wxxmlword)
 (defprop $%chi "<g>%chi</g>" wxxmlword)
+(defprop $chi "<g>chi</g>" wxxmlword)
 (defprop $%psi "<g>%psi</g>" wxxmlword)
+(defprop $psi "<g>psi</g>" wxxmlword)
 (defprop $%omega "<g>%omega</g>" wxxmlword)
+(defprop $omega "<g>omega</g>" wxxmlword)
 (defprop |$%Alpha| "<g>%Alpha</g>" wxxmlword)
+(defprop |$Alpha| "<g>Alpha</g>" wxxmlword)
 (defprop |$%Beta| "<g>%Beta</g>" wxxmlword)
+(defprop |$Beta| "<g>Beta</g>" wxxmlword)
 (defprop |$%Gamma| "<g>%Gamma</g>" wxxmlword)
+(defprop |$Gamma| "<g>Gamma</g>" wxxmlword)
 (defprop |$%Delta| "<g>%Delta</g>" wxxmlword)
+(defprop |$Delta| "<g>Delta</g>" wxxmlword)
 (defprop |$%Epsilon| "<g>%Epsilon</g>" wxxmlword)
+(defprop |$Epsilon| "<g>Epsilon</g>" wxxmlword)
 (defprop |$%Zeta| "<g>%Zeta</g>" wxxmlword)
+(defprop |$Zeta| "<g>Zeta</g>" wxxmlword)
 (defprop |$%Eta| "<g>%Eta</g>" wxxmlword)
+(defprop |$Eta| "<g>Eta</g>" wxxmlword)
 (defprop |$%Theta| "<g>%Theta</g>" wxxmlword)
+(defprop |$Theta| "<g>Theta</g>" wxxmlword)
 (defprop |$%Iota| "<g>%Iota</g>" wxxmlword)
+(defprop |$Iota| "<g>Iota</g>" wxxmlword)
 (defprop |$%Kappa| "<g>%Kappa</g>" wxxmlword)
+(defprop |$Kappa| "<g>Kappa</g>" wxxmlword)
 (defprop |$%Lambda| "<g>%Lambda</g>" wxxmlword)
+(defprop |$Lambda| "<g>Lambda</g>" wxxmlword)
 (defprop |$%Mu| "<g>%Mu</g>" wxxmlword)
+(defprop |$Mu| "<g>Mu</g>" wxxmlword)
 (defprop |$%Nu| "<g>%Nu</g>" wxxmlword)
+(defprop |$Nu| "<g>Nu</g>" wxxmlword)
 (defprop |$%Xi| "<g>%Xi</g>" wxxmlword)
+(defprop |$Xi| "<g>Xi</g>" wxxmlword)
 (defprop |$%Omicron| "<g>%Omicron</g>" wxxmlword)
+(defprop |$Omicron| "<g>Omicron</g>" wxxmlword)
 (defprop |$%Rho| "<g>%Rho</g>" wxxmlword)
+(defprop |$Rho| "<g>Rho</g>" wxxmlword)
 (defprop |$%Sigma| "<g>%Sigma</g>" wxxmlword)
+(defprop |$Sigma| "<g>Sigma</g>" wxxmlword)
 (defprop |$%Tau| "<g>%Tau</g>" wxxmlword)
+(defprop |$Tau| "<g>Tau</g>" wxxmlword)
 (defprop |$%Upsilon| "<g>%Upsilon</g>" wxxmlword)
+(defprop |$Upsilon| "<g>Upsilon</g>" wxxmlword)
 (defprop |$%Phi| "<g>%Phi</g>" wxxmlword)
+(defprop |$Phi| "<g>Phi</g>" wxxmlword)
 (defprop |$%Chi| "<g>%Chi</g>" wxxmlword)
+(defprop |$Chi| "<g>Chi</g>" wxxmlword)
 (defprop |$%Psi| "<g>%Psi</g>" wxxmlword)
+(defprop |$Psi| "<g>Psi</g>" wxxmlword)
 (defprop |$%Omega| "<g>%Omega</g>" wxxmlword)
+(defprop |$Omega| "<g>Omega</g>" wxxmlword)
 (defprop |$%Pi| "<g>%Pi</g>" wxxmlword)
+(defprop |$Pi| "<g>Pi</g>" wxxmlword)
 
 (defprop $%i "<s>%i</s>" wxxmlword)
 (defprop $%e "<s>%e</s>" wxxmlword)
@@ -346,7 +452,46 @@
 (defprop %product 95. wxxml-rbp)
 
 ;; insert left-angle-brackets for mncexpt. a^<t> is how a^^n looks.
+
 (defun wxxml-mexpt (x l r)
+  (cond ((atom (cadr x))
+	 (wxxml-mexpt-simple x l r))
+	((member 'array (caadr x))
+	 (wxxml-mexpt-array x l r))
+	(t
+	 (wxxml-mexpt-simple x l r))))
+
+(defun wxxml-mexpt-array (x l r)
+  (let* ((nc (eq (caar x) 'mncexpt))
+	 f (xarr (cadr x))
+	 (xexp (nformat (caddr x))))
+   ;; the index part
+    (if (eq 'mqapply (caar xarr))
+	(setq f (cadr xarr)
+	      xarr (cdr xarr)
+	      l (wxxml f (append l (list "<ie><p>")) (list "</p>")
+		       'mparen 'mparen))
+	(setq f (caar xarr)
+	      l (wxxml f (append l (if nc
+				       (list "<ie mat=\"true\"><r>")
+				       (list "<ie><r>")))
+		       (list "</r>") lop 'mfunction)))
+    (setq  l (append l (wxxml-list (cdr xarr) (list "<r>")
+				   (list "</r>") "<v>,</v>")))
+   ;; The exponent part
+    (setq r (if (mmminusp xexp)
+                ;; the change in base-line makes parens unnecessary
+                (wxxml (cadr xexp) '("</r><r>-")
+                       (cons "</r></ie>" r) 'mparen 'mparen)
+		(if (and (integerp xexp) (< xexp 10))
+		    (wxxml xexp nil
+			   (cons "</ie>" r) 'mparen 'mparen)
+		    (wxxml xexp (list "<r>")
+			   (cons "</r></ie>" r) 'mparen 'mparen)
+		    )))
+    (append l r)))
+
+(defun wxxml-mexpt-simple (x l r)
   (let((nc (eq (caar x) 'mncexpt)))
     (setq l (wxxml (cadr x) (append l (if nc
                                           '("<e mat=\"true\"><r>")
@@ -355,7 +500,7 @@
           r (if (mmminusp (setq x (nformat (caddr x))))
                 ;; the change in base-line makes parens unnecessary
                 (wxxml (cadr x) '("</r><r>-")
-                       (cons "</r></e>" r) 'mparen 'mparen)
+                       (cons "</r></e>" r) 'mminus 'mminus)
 		(if (and (integerp x) (< x 10))
 		    (wxxml x (list "</r>")
 			   (cons "</e>" r) 'mparen 'mparen)
@@ -400,16 +545,23 @@
                  (append '("</r></f>")r) 'mparen 'mparen))
   (append l r))
 
-(defprop $matrix wxxml-matrix wxxml)
+(defprop $matrix wxxml-matrix-test wxxml)
+
+(defun wxxml-matrix-test (x l r)
+  (if (every #'$listp (cdr x))
+      (wxxml-matrix x l r)
+      (wxxml-function x l r)))
 
 (defun wxxml-matrix(x l r) ;;matrix looks like ((mmatrix)((mlist) a b) ...)
   (cond ((null (cdr x))
-         (append l `("<fn><t>matrix</t><p/></fn>") r))
+         (append l `("<fn><fnm>matrix</fnm><p/></fn>") r))
         ((and (null (cddr x))
               (null (cdadr x)))
-         (append l `("<fn><t>matrix</t><p><t>[</t><t>]</t></p></fn>") r))
+         (append l `("<fn><fnm>matrix</fnm><p><t>[</t><t>]</t></p></fn>") r))
         (t
-         (append l `("<tb>")
+         (append l (if (find 'inference (car x))
+		       (list "<tb inference='t'>")
+		       (list "<tb>"))
                  (mapcan #'(lambda (y)
 			     (cond ((null (cdr y))
 				    (list "<mtr><mtd><mspace/></mtd></mtr>"))
@@ -427,11 +579,14 @@
 (defprop %sum wxxml-sum wxxml)
 (defprop %lsum wxxml-lsum wxxml)
 (defprop %product wxxml-sum wxxml)
+(defprop $sum wxxml-sum wxxml)
+(defprop $lsum wxxml-lsum wxxml)
+(defprop $product wxxml-sum wxxml)
 
 ;; easily extended to union, intersect, otherops
 
 (defun wxxml-lsum(x l r)
-  (let ((op (cond ((eq (caar x) '%lsum) "<sm><r>")))
+  (let ((op "<sm><r>")
 	;; gotta be one of those above
 	(s1 (wxxml (cadr x) nil nil 'mparen rop));; summand
 	(index ;; "index = lowerlimit"
@@ -442,11 +597,10 @@
 		,@s1 "</r></sm>") r)))
 
 (defun wxxml-sum(x l r)
-  (let ((op (cond ((eq (caar x) '%sum) "<sm><r>")
-		  ((eq (caar x) '%product) "<sm type=\"prod\"><r>")
-                  ;; extend here
-		  ))
-	;; gotta be one of those above
+  (let ((op (if (or (eq (caar x) '%sum)
+		    (eq (caar x) '$sum))
+		"<sm><r>"
+		"<sm type=\"prod\"><r>"))
 	(s1 (wxxml (cadr x) nil nil 'mparen rop));; summand
 	(index ;; "index = lowerlimit"
          (wxxml `((mequal simp) ,(caddr x) ,(cadddr x))
@@ -457,6 +611,7 @@
 		,@s1 "</r></sm>") r)))
 
 (defprop %integrate wxxml-int wxxml)
+(defprop $integrate wxxml-int wxxml)
 
 (defun wxxml-int (x l r)
   (let ((s1 (wxxml (cadr x) nil nil 'mparen 'mparen));;integrand delims / & d
@@ -492,9 +647,19 @@
 	(subfun ;; the thing underneath "limit"
          (wxxml `((mrarr simp) ,(third x)
                   ,(fourth x)) nil nil 'mparen 'mparen)))
-    (append l `("<lm><t>lim</t><r>"
-                ,@subfun "</r><r>"
-                ,@s1 "</r></lm>") r)))
+    (case (fifth x)
+      ($plus
+       (append l `("<lm><fnm>lim</fnm><r>"
+		   ,@subfun "<t>+</t></r><r>"
+		   ,@s1 "</r></lm>") r))
+      ($minus
+       (append l `("<lm><fnm>lim</fnm><r>"
+		   ,@subfun "<t>-</t></r><r>"
+		   ,@s1 "</r></lm>") r))
+      (otherwise
+       (append l `("<lm><fnm>lim</fnm><r>"
+		   ,@subfun "</r><r>"
+		   ,@s1 "</r></lm>") r)))))
 
 (defprop %at wxxml-at wxxml)
 ;; e.g.  at(diff(f(x)),x=a)
@@ -529,10 +694,11 @@
 (defprop mplus 100. wxxml-rbp)
 
 (defun wxxml-mplus (x l r)
-  (cond ((memq 'trunc (car x))(setq r (cons "<t>+</t><t>...</t>" r))))
+  (cond ((member 'trunc (car x) :test #'eq)
+	 (setq r (cons "<t>+</t><t>...</t>" r))))
   (cond ((null (cddr x))
          (if (null (cdr x))
-             (wxxml-function x l r t)
+             (wxxml-function x l r)
 	     (wxxml (cadr x) l r 'mplus rop)))
         (t (setq l (wxxml (cadr x) l nil lop 'mplus)
                  x (cddr x))
@@ -552,8 +718,8 @@
 (defprop mminus wxxml-prefix wxxml)
 (defprop mminus ("-") wxxmlsym)
 (defprop mminus "<t>-</t>" wxxmlword)
-(defprop mminus 100. wxxml-rbp)
-(defprop mminus 100. wxxml-lbp)
+(defprop mminus 101. wxxml-rbp)
+(defprop mminus 101. wxxml-lbp)
 
 (defprop $~ wxxml-infix wxxml)
 (defprop $~ ("<t>~</t>") wxxmlsym)
@@ -562,8 +728,8 @@
 (defprop $~ 133. wxxml-rbp)
 
 (defprop min wxxml-infix wxxml)
-(defprop min ("<t>in</t>") wxxmlsym)
-(defprop min "<t>in</t>" wxxmlword)
+(defprop min ("<fnm>in</fnm>") wxxmlsym)
+(defprop min "<fnm>in</fnm>" wxxmlword)
 (defprop min 80. wxxml-lbp)
 (defprop min 80. wxxml-rbp)
 
@@ -574,6 +740,7 @@
 (defprop mequal 80. wxxml-rbp)
 
 (defprop mnotequal wxxml-infix wxxml)
+(defprop mnotequal ("<t>#</t>") wxxmlsym)
 (defprop mnotequal 80. wxxml-lbp)
 (defprop mnotequal 80. wxxml-rbp)
 
@@ -602,81 +769,23 @@
 (defprop mleqp 80. wxxml-rbp)
 
 (defprop mnot wxxml-prefix wxxml)
-(defprop mnot ("<t>not</t>") wxxmlsym)
-(defprop mnot "<t>not</t>" wxxmlword)
+(defprop mnot ("<fnm>not</fnm>") wxxmlsym)
+(defprop mnot "<fnm>not</fnm>" wxxmlword)
 (defprop mnot 70. wxxml-rbp)
 
 (defprop mand wxxml-nary wxxml)
-(defprop mand "<mspace/><t>and</t><mspace/>" wxxmlsym)
-(defprop mand "<t>and</t>" wxxmlword)
+(defprop mand "<mspace/><fnm>and</fnm><mspace/>" wxxmlsym)
+(defprop mand "<fnm>and</fnm>" wxxmlword)
 (defprop mand 60. wxxml-lbp)
 (defprop mand 60. wxxml-rbp)
 
 (defprop mor wxxml-nary wxxml)
-(defprop mor "<mspace/><t>or</t><mspace/>" wxxmlsym)
-(defprop mor "<t>or</t>" wxxmlword)
+(defprop mor "<mspace/><fnm>or</fnm><mspace/>" wxxmlsym)
+(defprop mor "<fnm>or</fnm>" wxxmlword)
 (defprop mor 50. wxxml-lbp)
 (defprop mor 50. wxxml-rbp)
 
 
-(defun wxxml-setup (x)
-  (let((a (car x))
-       (b (cadr x)))
-    ;;      (setf (get a 'wxxml) 'wxxml-prefix) ; we don't want sin x
-    (setf (get a 'wxxmlword) b)
-    (setf (get a 'wxxmlsym) (list b))
-    (setf (get a 'wxxml-rbp) 320)
-    (setf (get a 'wxxml-lbp) 320)))
-
-
-(mapc #'wxxml-setup
-      '(
-        (%acos "<v>acos</v>")
-        (%asin "<v>asin</v>")
-        (%asinh "<v>asinh</v>")
-        (%acosh "<v>acosh</v>")
-        (%atan "<v>atan</v>")
-        (%atanh "<v>atanh</v>")
-        (%arg "<v>arg</v>")
-        (%bessel_j "<v>bessel_j</v>")
-        (%bessel_i "<v>bessel_i</v>")
-        (%bessel_k "<v>bessel_k</v>")
-        (%bessel_y "<v>bessel_y</v>")
-        (%beta "<v>beta</v>")
-        (%cos "<v>cos</v>")
-        (%cosh "<v>cosh</v>")
-        (%cot "<v>cot</v>")
-        (%coth "<v>coth</v>")
-        (%csc "<v>csc</v>")
-        (%deg "<v>deg</v>")
-        (%determinant "<v>determinant</v>")
-        (%dim "<v>dim</v>")
-        (%exp "<v>exp</v>")
-        (%gamma "<g>gamma</g>")
-        (%gcd "<v>gcd</v>")
-        (%hom "<v>hom</v>")
-        (%ker "<v>ker</v>")
-        (%lg "<v>lg</v>")
-        (%liminf "<v>lim inf</v>")
-        (%limsup "<v>lim sup</v>")
-        (%ln "<v>ln</v>")
-        ($li "<v>li</v>")
-        (%log "<v>log</v>")
-        (%max "<v>max</v>")
-        (%min "<v>min</v>")
-        ($min "<v>min</v>")
-        ($psi "<v>psi</v>")
-        (%sec "<v>sec</v>")
-        (%sech "<v>sech</v>")
-        (%sin "<v>sin</v>")
-        (%sinh "<v>sinh</v>")
-        (%sup "<v>sup</v>")
-        (%tan "<v>tan</v>")
-        (%tanh "<v>tanh</v>")
-        (%erf "<v>erf</v>")
-        (%laplace "<v>laplace</v>")
-        ))
-
 (defprop mcond wxxml-mcond wxxml)
 (defprop mcond 25. wxxml-lbp)
 (defprop mcond 25. wxxml-rbp)
@@ -686,14 +795,40 @@
 (defprop %derivative 119. wxxml-rbp)
 
 (defun wxxml-derivative (x l r)
-  (wxxml (wxxml-d x "<s>d</s>") (append l '("<d>"))
-         (append '("</d>") r) 'mparen 'mparen))
+  (if $derivabbrev
+      (append l
+	      (wxxml-d-abbrev x)
+	      r)
+      (wxxml (wxxml-d x) (append l '("<d>"))
+	     (append '("</d>") r) 'mparen 'mparen)))
 
-(defun wxxml-d (x dsym) ;dsym should be "&DifferentialD;" or "&PartialD;"
+(defun wxxml-d-abbrev-subscript (l_vars l_ords &aux var_xml)
+  (let ((sub ()))
+    (loop while l_vars do
+	  (setq var_xml (car (wxxml (car l_vars) nil nil 'mparen 'mparen)))
+	  (loop for i from 1 to (car l_ords) do
+		(setq sub (cons var_xml sub)))
+	  (setq l_vars (cdr l_vars)
+		l_ords (cdr l_ords)))
+    (reverse sub)))
+
+(defun wxxml-d-abbrev (x)
+  (let*
+      ((difflist (cddr x))
+       (ords (odds  difflist 0))
+       (ords (cond ((null ords) '(1))
+		   (t ords)))
+       (vars (odds difflist 1))
+       (fun (wxxml (cadr x) nil nil 'mparen 'mparen)))
+    (append '("<i d=\"1\"><r>") fun '("</r>")
+	    '("<r>") (wxxml-d-abbrev-subscript vars ords) '("</r></i>"))))
+
+(defun wxxml-d (x)
   ;; format the macsyma derivative form so it looks
   ;; sort of like a quotient times the deriva-dand.
   (let*
       (($simp t)
+       (dsym '((wxxmltag simp) "d" "s"))
        (arg (cadr x)) ;; the function being differentiated
        (difflist (cddr x)) ;; list of derivs e.g. (x 1 y 2)
        (ords (odds difflist 0)) ;; e.g. (1 2)
@@ -709,19 +844,10 @@
       ((mquotient) ,(simplifya numer nil) ,denom)
       ,arg)))
 
-;;(defun wxxml-mcond (x l r)
-;;  (append l
-;;          (wxxml (cadr x) '("<t>if</t><mspace/>")
-;;                 '("<mspace/><t>then</t><mspace/>") 'mparen 'mparen)
-;;          (if (eql (fifth x) '$false)
-;;              (wxxml (caddr x) nil r 'mcond rop)
-;;            (append (wxxml (caddr x) nil nil 'mparen 'mparen)
-;;                    (wxxml (fifth x) '("<mspace/><t>else</t><mspace/>")
-;;                           r 'mcond rop)))))
 (defun wxxml-mcond (x l r)
   (let ((res ()))
-    (setq res (wxxml (cadr x) '("<t>if</t><mspace/>")
-		     '("<mspace/><t>then</t><mspace/>") 'mparen 'mparen))
+    (setq res (wxxml (cadr x) '("<fnm>if</fnm><mspace/>")
+		     '("<mspace/><fnm>then</fnm><mspace/>") 'mparen 'mparen))
     (setq res (append res (wxxml (caddr x) nil
 				 '("<mspace/>") 'mparen 'mparen)))
     (let ((args (cdddr x)))
@@ -730,13 +856,13 @@
 	      ((and (= (length args) 2) (eql (car args) t))
 	       (unless (or (eql (cadr args) '$false) (null (cadr args)))
 		 (setq res (wxxml (cadr args)
-				  (append res '("<t>else</t><mspace/>"))
+				  (append res '("<fnm>else</fnm><mspace/>"))
 				  nil 'mparen 'mparen))))
 	      (t
 	       (setq res (wxxml (car args)
-				(append res '("<t>elseif</t><mspace/>"))
+				(append res '("<fnm>elseif</fnm><mspace/>"))
 				(wxxml (cadr args)
-				       '("<mspace/><t>then</t><mspace/>")
+				       '("<mspace/><fnm>then</fnm><mspace/>")
 				       '("<mspace/>") 'mparen 'mparen)
 				'mparen 'mparen))))
 	    (setq args (cddr args)))
@@ -765,28 +891,32 @@
   (wxxml-list (wxxmlmdoin x) l r "<mspace/>"))
 
 (defun wxxmlmdo (x)
-  (nconc (cond ((second x) `("<t>for</t>" ,(second x))))
+  (nconc (cond ((second x) (list (make-tag "for" "fnm") (second x))))
 	 (cond ((equal 1 (third x)) nil)
-	       ((third x)  `("<t>from</t>" ,(third x))))
+	       ((third x)  (list (make-tag "from" "fnm") (third x))))
 	 (cond ((equal 1 (fourth x)) nil)
-	       ((fourth x) `("<t>step</t>" ,(fourth x)))
-	       ((fifth x)  `("<t>next</t>" ,(fifth x))))
-	 (cond ((sixth x)  `("<t>thru</t>" ,(sixth x))))
+	       ((fourth x)
+		(list (make-tag "step" "fnm")  (fourth x)))
+	       ((fifth x)
+		(list (make-tag "next" "fnm") (fifth x))))
+	 (cond ((sixth x)
+		(list (make-tag "thru" "fnm") (sixth x))))
 	 (cond ((null (seventh x)) nil)
 	       ((eq 'mnot (caar (seventh x)))
-		`("<t>while</t>" ,(cadr (seventh x))))
-	       (t `("<t>unless</t>" ,(seventh x))))
-	 `("<t>do</t>" ,(eighth x))))
+		(list (make-tag "while" "fnm") (cadr (seventh x))))
+	       (t (list (make-tag "unless" "fnm") (seventh x))))
+	 (list (make-tag "do" "fnm") (eighth x))))
 
 (defun wxxmlmdoin (x)
-  (nconc `("<t>for</t>" ,(second x) "<t>in</t>"
-           ,(third x))
-	 (cond ((sixth x) `("<t>thru</t>" ,(sixth x))))
+  (nconc (list (make-tag "for" "fnm") (second x)
+	       (make-tag "in" "fnm") (third x))
+	 (cond ((sixth x)
+		(list (make-tag "thru" "fnm") (sixth x))))
 	 (cond ((null (seventh x)) nil)
 	       ((eq 'mnot (caar (seventh x)))
-		`("<t>while</t>" ,(cadr (seventh x))))
-	       (t `("<t>unless</t>" ,(seventh x))))
-	 `("<t>do</t>" ,(eighth x))))
+		(list (make-tag "while" "fnm") (cadr (seventh x))))
+	       (t (list (make-tag "unless" "fnm") (seventh x))))
+	 (list (make-tag "do" "fnm") (eighth x))))
 
 
 (defun wxxml-matchfix-np (x l r)
@@ -821,10 +951,8 @@
 (defprop spaceout wxxml-spaceout wxxml)
 
 (defun mydispla (x)
-  (let ((ccol 1)
-        (texport *standard-output*))
-    (mapc #'princ
-          (wxxml x '("<mth>") '("</mth>") 'mparen 'mparen))))
+  (mapc #'princ
+	(wxxml x '("<mth>") '("</mth>") 'mparen 'mparen)))
 
 (setf *alt-display2d* 'mydispla)
 
@@ -843,6 +971,22 @@
      (setq tp '$unknown)))
   tp)
 
+(defun wxxml-inference (x l r)
+  (let ((name (cadr x))
+	(values (caddr x))
+	(dis (cadddr x))
+	(m ()))
+    (labels
+	((build-eq (e)
+	   `((mequal simp) ,(cadr e) ,(caddr e))))
+      (dolist (i (cdr dis))
+	(setq m (append m `(((mlist simp) ,(build-eq (nth i values)))))))
+      (setq m (cons `((mlist simp) ,name) m))
+      (setq m (cons '($matrix simp inference) m))
+      (wxxml m l r 'mparen 'mparen))))
+
+(defprop $inference_result wxxml-inference wxxml)
+
 ;;;
 ;;; This is the display support only - copy/paste will not work
 ;;;
@@ -853,12 +997,13 @@
 (defmvar $pdiff_diff_var_names (list '(mlist) '|$x| '|$y| '|$z|))
 
 (setf (get '%pderivop 'wxxml) 'wxxml-pderivop)
+(setf (get '$pderivop 'wxxml) 'wxxml-pderivop)
 
 (defun wxxml-pderivop (x l r)
   (cond ((and $pdiff_uses_prime_for_derivatives (eq 3 (length x)))
 	 (let* ((n (car (last x)))
 		(p))
-	   
+
 	   (cond ((<= n $pdiff_prime_limit)
 		  (setq p (make-list n :initial-element "'")))
 		 (t
@@ -873,7 +1018,7 @@
                           (wxxml (cadr x) nil nil lop rop)
 			  (list "</r><r>") p
                           (list "</r></e>")  r)))))
-	
+
 	((and $pdiff_uses_named_subscripts_for_derivatives
 	      (< (apply #'+ (cddr x)) $pdiff_prime_limit))
 	 (let ((n (cddr x))
@@ -894,18 +1039,242 @@
 		 (wxxml-list (cddr x) nil nil ",")
                  (list ")</r></i>") r))))
 
+;;
+;; Plotting support
+;;
+
+(defprop wxxmltag wxxml-tag wxxml)
+
+(defun wxxml-tag (x l r)
+  (let ((name (cadr x))
+	(tag (caddr x))
+	(prop (cadddr x)))
+    (if prop
+	(append l (list (format nil "<~a ~a>~a</~a>" tag prop name tag)) r)
+	(append l (list (format nil "<~a>~a</~a>" tag name tag)) r))))
+
+
+(defmvar $wxplot_size '((mlist simp) 500 300))
+(defmvar $wxplot_old_gnuplot nil)
+
+(defvar *image-counter* 0)
+
+(defun wxplot-filename (&optional (suff t))
+  (incf *image-counter*)
+  (plot-temp-file (if suff
+		      (format nil "maxout_~d.png" *image-counter*)
+		      (format nil "maxout_~d" *image-counter*))))
+
+(defun $wxplot_preamble ()
+  (let ((frmt (if $wxplot_old_gnuplot
+		  "set terminal png picsize ~d ~d; set zeroaxis;"
+		  "set terminal png size ~d,~d; set zeroaxis;")))
+    (format nil frmt
+	    ($first $wxplot_size)
+	    ($second $wxplot_size))))
+
+(defun $with_slider (&rest args)
+  (apply #'$wxanimate args))
+
+(defun $wxanimate (a a-range expr &rest args)
+  (let (images)
+    (dolist (aval (reverse (cdr a-range)))
+      (let ((preamble ($wxplot_preamble))
+	    (system-preamble (get-plot-option-string '$gnuplot_preamble 2))
+	    (filename (wxplot-filename))
+	    (expr (maxima-substitute aval a expr)))
+	(setq preamble
+	      (format nil "set title '~a=~a'~%~a"
+		      (stripdollar (maybe-invert-string-case (symbol-name a)))
+		      (mfuncall '$string aval)
+		      preamble))
+	(setq preamble (format nil "~a; ~a" preamble system-preamble))
+	(dolist (arg args)
+	  (if (and (listp arg) (eql (cadr arg) '$gnuplot_preamble))
+	      (setq preamble (format nil "~a; ~a"
+				     preamble (caddr arg)))))
+	(apply #'$plot2d `(,expr ,@args
+			   ((mlist simp) $plot_format $gnuplot)
+			   ((mlist simp) $gnuplot_preamble ,preamble)
+			   ((mlist simp) $gnuplot_term $png)
+			   ((mlist simp) $gnuplot_out_file ,filename)))
+	(setq images (cons filename images))))
+    ($ldisp (list '(wxxmltag simp) (format nil "~{~a;~}" images) "slide")))
+  "")
+
+(defvar *windows-OS* (string= *autoconf-win32* "true"))
+
+(defun $with_slider_draw (&rest args)
+  (apply #'$wxanimate_draw args))
+
+(defun draw-transform (arg trans)
+  (declare (ignore trans))
+  arg)
+
+(defun $wxanimate_draw (a a-range &rest args)
+  (let (images)
+    (dolist (aval (reverse (cdr a-range)))
+      (let* ((filename (wxplot-filename nil))
+	     (*windows-OS* t)
+	     (args (cons `((mequal simp) $title
+			   ,(format nil "~a=~a"
+				   (stripdollar (maybe-invert-string-case (symbol-name a)))
+				   (mfuncall '$string aval)))
+			 args))
+	     (args (cons '($gr2d)
+			 (draw-transform
+			  (mapcar #'(lambda (arg) (maxima-substitute aval a arg))
+				  args)
+			  '$draw2d_transform))))
+	(setq images (cons (format nil "~a.png" filename) images))
+	($apply '$draw
+		(append
+		 `((mlist simp)
+		   ((mequal simp) $terminal $png)
+		   ((mequal simp) $pic_width ,($first $wxplot_size))
+		   ((mequal simp) $pic_height ,($second $wxplot_size))
+		   ((mequal simp) $file_name ,filename))
+		 (list args)))))
+    ($ldisp (list '(wxxmltag simp) (format nil "~{~a;~}" images) "slide")))
+  "")
+
+(defun $with_slider_draw3d (&rest args)
+  (apply #'$wxanimate_draw3d args))
+
+(defun $wxanimate_draw3d (a a-range &rest args)
+  (let (images)
+    (dolist (aval (reverse (cdr a-range)))
+      (let* ((filename (wxplot-filename nil))
+	     (*windows-OS* t)
+	     (args (cons `((mequal simp) $title
+			   ,(format nil "~a=~a"
+				   (stripdollar (maybe-invert-string-case (symbol-name a)))
+				   (mfuncall '$string aval)))
+			 args))
+	     (args (cons '($gr3d)
+			 (draw-transform
+			  (mapcar #'(lambda (arg) (maxima-substitute aval a arg))
+				  args)
+			  '$draw3d_transform))))
+	(setq images (cons (format nil "~a.png" filename) images))
+	($apply '$draw
+		(append
+		 `((mlist simp)
+		   ((mequal simp) $terminal $png)
+		   ((mequal simp) $pic_width ,($first $wxplot_size))
+		   ((mequal simp) $pic_height ,($second $wxplot_size))
+		   ((mequal simp) $file_name ,filename))
+		 (list args)))))
+    ($ldisp (list '(wxxmltag simp) (format nil "~{~a;~}" images) "slide")))
+  "")
+
+(defun $wxplot2d (&rest args)
+  (let ((preamble ($wxplot_preamble))
+	(system-preamble (get-plot-option-string '$gnuplot_preamble 2))
+	(filename (wxplot-filename)))
+    (setq preamble (format nil "~a; ~a" preamble system-preamble))
+    (dolist (arg args)
+      (if (and (listp arg) (eql (cadr arg) '$gnuplot_preamble))
+	  (setq preamble (format nil "~a; ~a"
+				 preamble (caddr arg)))))
+    (apply #'$plot2d `(,@args
+		       ((mlist simp) $plot_format $gnuplot)
+		       ((mlist simp) $gnuplot_preamble ,preamble)
+		       ((mlist simp) $gnuplot_term $png)
+		       ((mlist simp) $gnuplot_out_file ,filename)))
+    ($ldisp `((wxxmltag simp) ,filename "img")))
+  "")
+
+(defun $wxplot3d (&rest args)
+  (let ((preamble ($wxplot_preamble))
+	(system-preamble (get-plot-option-string '$gnuplot_preamble 2))
+	(filename (wxplot-filename)))
+    (setq preamble (format nil "~a; ~a" preamble system-preamble))
+    (dolist (arg args)
+      (if (and (listp arg) (eql (cadr arg) '$gnuplot_preamble))
+	  (setq preamble (format nil "~a; ~a"
+				 preamble (caddr arg)))))
+    (apply #'$plot3d `(,@args
+		       ((mlist simp) $plot_format $gnuplot)
+		       ((mlist simp) $gnuplot_preamble ,preamble)
+		       ((mlist simp) $gnuplot_term $png)
+		       ((mlist simp) $gnuplot_out_file ,filename)))
+    ($ldisp `((wxxmltag simp) ,filename "img")))
+  "")
+
+
+(setf (get '$draw 'autoload) "draw")
+
+(defun $wxdraw2d (&rest args)
+  (apply #'$wxdraw
+	 (list (cons '($gr2d) (draw-transform args '$draw2d_transform)))))
+
+(defun $wxdraw3d (&rest args)
+  (apply #'$wxdraw
+	 (list (cons '($gr3d) (draw-transform args '$draw3d_transform)))))
+
+(defun $wxdraw (&rest args)
+  (let* ((filename (wxplot-filename nil))
+	 (*windows-OS* t)
+	 res)
+    (setq res ($apply '$draw
+		     (append
+		      `((mlist simp)
+			((mequal simp) $terminal $png)
+			((mequal simp) $pic_width ,($first $wxplot_size))
+			((mequal simp) $pic_height ,($second $wxplot_size))
+			((mequal simp) $file_name ,filename))
+		      args)))
+    ($ldisp `((wxxmltag simp) ,(format nil "~a.png" filename) "img"))
+    res))
+
+(defun $wximplicit_plot (&rest args)
+  (let ((preamble ($wxplot_preamble))
+	(system-preamble (get-plot-option-string '$gnuplot_preamble 2))
+	(filename (wxplot-filename)))
+    (setq preamble (format nil "~a; ~a" preamble system-preamble))
+    (dolist (arg args)
+      (if (and (listp arg) (eql (cadr arg) '$gnuplot_preamble))
+	  (setq preamble (format nil "~a; ~a"
+				 preamble (caddr arg)))))
+    ($apply '$implicit_plot `((mlist simp) ,@args
+			      ((mlist simp) $plot_format $gnuplot)
+			      ((mlist simp) $gnuplot_preamble ,preamble)
+			      ((mlist simp) $gnuplot_term $png)
+			      ((mlist simp) $gnuplot_out_file ,filename)))
+    ($ldisp `((wxxmltag simp) ,filename "img")))
+  "")
+
+
+(defun $wxcontour_plot (&rest args)
+  (let ((preamble ($wxplot_preamble))
+	(system-preamble (get-plot-option-string '$gnuplot_preamble 2))
+	(filename (wxplot-filename)))
+    (setq preamble (format nil "~a; ~a" preamble system-preamble))
+    (dolist (arg args)
+      (if (and (listp arg) (eql (cadr arg) '$gnuplot_preamble))
+	  (setq preamble (format nil "~a; ~a" preamble (caddr arg)))))
+    (apply #'$contour_plot `(,@args
+			     ((mlist simp) $plot_format $gnuplot)
+			     ((mlist simp) $gnuplot_preamble ,preamble)
+			     ((mlist simp) $gnuplot_term $png)
+			     ((mlist simp) $gnuplot_out_file ,filename)))
+
+    ($ldisp `((wxxmltag simp) ,filename "img")))
+  "")
+
+(defun $show_image (file)
+ ($ldisp `((wxxmltag simp) ,file "img" "del=\"no\"")))
 
 ;;
 ;; Port of Barton Willis's texput function.
 ;;
 
 (defun $wxxmlput (e s &optional tx lbp rbp)
-  (cond ((mstringp e)
-	 (setq e (define-symbol (string-left-trim '(#\&) e)))))
   (cond (($listp s)
 	 (setq s (margs s)))
 	(t
-	 (setq s (list s))))
+	 (setq s (list ($sconcat s)))))
   (setq s (mapcar #'wxxml-stripdollar s))
   (cond ((or (null lbp) (not (integerp lbp)))
          (setq lbp 180)))
