@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2006 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -33,6 +33,7 @@ public:
   void SelectInner(wxRect& rect, MathCell** first, MathCell** last);
   bool BreakUp();
   void Unbreak(bool all);
+  void SetParent(MathCell *parent, bool all);
 protected:
   MathCell *m_innerCell;
   MathCell *m_open, *m_close, *m_last;
@@ -40,6 +41,9 @@ protected:
   void RecalculateWidths(CellParser& parser, int fontsize, bool all);
   void Draw(CellParser& parser, wxPoint point, int fontsize, bool all);
   wxString ToString(bool all);
+  wxString ToTeX(bool all);
+
+  wxString ToXML(bool all);	//new!!!
 };
 
 #endif //_ABSCELL_H_
