@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2006 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -35,7 +35,10 @@ public:
   void RecalculateWidths(CellParser& parser, int fontsize, bool all);
   void Draw(CellParser& parser, wxPoint point, int fontsize, bool all);
   wxString ToString(bool all);
+  wxString ToTeX(bool all);
+	wxString ToXML(bool all);	//new!!!
   void SelectInner(wxRect& rect, MathCell** first, MathCell** last);
+  void SetParent(MathCell *parent, bool all);
 protected:
   MathCell *m_baseCell;
   MathCell *m_indexCell;
