@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2006 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -23,20 +23,19 @@ BC2Wiz::BC2Wiz(wxWindow* parent, int id, const wxString& title,
                const wxPoint& pos, const wxSize& size, long style):
     wxDialog(parent, id, title, pos, size, wxDEFAULT_DIALOG_STYLE)
 {
-  label_1 = new wxStaticText(this, -1, _("BC2"));
   label_2 = new wxStaticText(this, -1, _("Solution:"));
   text_ctrl_1 = new BTextCtrl(this, -1, wxEmptyString, wxDefaultPosition,
                               wxSize(230, -1));
-  label_3 = new wxStaticText(this, -1, _("At point:"));
+  label_3 = new wxStaticText(this, -1, _("Point:"));
   text_ctrl_2 = new BTextCtrl(this, -1, wxT("x="), wxDefaultPosition,
                               wxSize(70, -1));
-  label_4 = new wxStaticText(this, -1, _("the value is:"));
+  label_4 = new wxStaticText(this, -1, _("Value:"));
   text_ctrl_3 = new BTextCtrl(this, -1, wxT("y="), wxDefaultPosition,
                               wxSize(70, -1));
-  label_5 = new wxStaticText(this, -1, _("At point:"));
+  label_5 = new wxStaticText(this, -1, _("Point:"));
   text_ctrl_4 = new BTextCtrl(this, -1, wxT("x="), wxDefaultPosition,
                               wxSize(70, -1));
-  label_6 = new wxStaticText(this, -1, _("the value is:"));
+  label_6 = new wxStaticText(this, -1, _("Value:"));
   text_ctrl_5 = new BTextCtrl(this, -1, wxT("y="), wxDefaultPosition,
                               wxSize(70, -1));
   static_line_1 = new wxStaticLine(this, -1);
@@ -56,40 +55,40 @@ BC2Wiz::BC2Wiz(wxWindow* parent, int id, const wxString& title,
 void BC2Wiz::set_properties()
 {
   SetTitle(_("BC2"));
-  label_1->SetFont(wxFont(20, wxROMAN, wxITALIC, wxNORMAL, 0, wxEmptyString));
 #if defined __WXMSW__
   button_1->SetDefault();
 #else
   button_2->SetDefault();
 #endif
+
+  text_ctrl_1->SetFocus();
 }
 
 
 void BC2Wiz::do_layout()
 {
-  wxFlexGridSizer* grid_sizer_1 = new wxFlexGridSizer(4, 1, 3, 3);
+  wxFlexGridSizer* grid_sizer_1 = new wxFlexGridSizer(4, 1, 0, 0);
   wxBoxSizer* sizer_1 = new wxBoxSizer(wxHORIZONTAL);
-  wxFlexGridSizer* grid_sizer_2 = new wxFlexGridSizer(2, 2, 3, 3);
+  wxFlexGridSizer* grid_sizer_2 = new wxFlexGridSizer(2, 2, 0, 0);
   wxBoxSizer* sizer_2 = new wxBoxSizer(wxHORIZONTAL);
   wxBoxSizer* sizer_3 = new wxBoxSizer(wxHORIZONTAL);
-  grid_sizer_1->Add(label_1, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 2);
-  grid_sizer_2->Add(label_2, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  grid_sizer_2->Add(text_ctrl_1, 0, wxALL, 2);
-  grid_sizer_2->Add(label_3, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  sizer_1->Add(text_ctrl_2, 0, wxALL, 2);
-  sizer_1->Add(label_4, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  sizer_1->Add(text_ctrl_3, 0, wxALL, 2);
+  grid_sizer_2->Add(label_2, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  grid_sizer_2->Add(text_ctrl_1, 0, wxALL, 5);
+  grid_sizer_2->Add(label_3, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  sizer_1->Add(text_ctrl_2, 0, wxALL, 5);
+  sizer_1->Add(label_4, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  sizer_1->Add(text_ctrl_3, 0, wxALL, 5);
   grid_sizer_2->Add(sizer_1, 0, wxALL, 0);
-  grid_sizer_2->Add(label_5, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  sizer_2->Add(text_ctrl_4, 0, wxALL, 2);
-  sizer_2->Add(label_6, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  sizer_2->Add(text_ctrl_5, 0, wxALL, 2);
+  grid_sizer_2->Add(label_5, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  sizer_2->Add(text_ctrl_4, 0, wxALL, 5);
+  sizer_2->Add(label_6, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  sizer_2->Add(text_ctrl_5, 0, wxALL, 5);
   grid_sizer_2->Add(sizer_2, 0, wxALL, 0);
   grid_sizer_1->Add(grid_sizer_2, 1, 0, 0);
   grid_sizer_1->Add(static_line_1, 0, wxEXPAND | wxLEFT | wxRIGHT, 2);
-  sizer_3->Add(button_1, 0, wxLEFT | wxRIGHT, 5);
-  sizer_3->Add(button_2, 0, wxLEFT | wxRIGHT, 5);
-  grid_sizer_1->Add(sizer_3, 1, wxALIGN_RIGHT | wxTOP | wxBOTTOM, 3);
+  sizer_3->Add(button_1, 0, wxALL, 5);
+  sizer_3->Add(button_2, 0, wxALL, 5);
+  grid_sizer_1->Add(sizer_3, 1, wxALIGN_RIGHT, 0);
   SetAutoLayout(true);
   SetSizer(grid_sizer_1);
   grid_sizer_1->Fit(this);
