@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2006 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -52,8 +52,13 @@ BTextCtrl::~BTextCtrl()
 
 void BTextCtrl::OnChar(wxKeyEvent& event)
 {
+#if wxUSE_UNICODE
+ if (!m_matchParens || MatchParenthesis(event.GetUnicodeKey()))
+    event.Skip();
+#else
   if (!m_matchParens || MatchParenthesis(event.GetKeyCode()))
     event.Skip();
+#endif
 }
 
 bool BTextCtrl::MatchParenthesis(int code)
@@ -62,19 +67,31 @@ bool BTextCtrl::MatchParenthesis(int code)
   switch (code)
   {
   case '(':
-    CloseParenthesis(wxT("("), wxT(")"));
+    CloseParenthesis(wxT("("), wxT(")"), true);
+    skip = false;
+    break;
+  case ')':
+    CloseParenthesis(wxT("("), wxT(")"), false);
     skip = false;
     break;
   case '[':
-    CloseParenthesis(wxT("["), wxT("]"));
+    CloseParenthesis(wxT("["), wxT("]"), true);
+    skip = false;
+    break;
+  case ']':
+    CloseParenthesis(wxT("["), wxT("]"), false);
     skip = false;
     break;
   case '{':
-    CloseParenthesis(wxT("{"), wxT("}"));
+    CloseParenthesis(wxT("{"), wxT("}"), true);
+    skip = false;
+    break;
+  case '}':
+    CloseParenthesis(wxT("{"), wxT("}"), false);
     skip = false;
     break;
   case '"':
-    CloseParenthesis(wxT("\""), wxT("\""));
+    CloseParenthesis(wxT("\""), wxT("\""), true);
     skip = false;
     break;
   case WXK_UP:
@@ -88,7 +105,7 @@ bool BTextCtrl::MatchParenthesis(int code)
   return skip;
 }
 
-void BTextCtrl::CloseParenthesis(wxString open, wxString close)
+void BTextCtrl::CloseParenthesis(wxString open, wxString close, bool fromOpen)
 {
   long from, to;
   GetSelection(&from, &to);
@@ -104,7 +121,10 @@ void BTextCtrl::CloseParenthesis(wxString open, wxString close)
     WriteText(close);
     SetInsertionPoint(from);
     WriteText(open);
-    SetInsertionPoint(to + 1);
+    if (fromOpen)
+      SetInsertionPoint(from);
+    else
+      SetInsertionPoint(to+2);
   }
 }
 
