@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2006 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -36,10 +36,15 @@ public:
   {
     m_matchParens = match;
   }
+  void SetSkipTab(bool skip)
+  {
+    m_skipTab = skip;
+  }
 private:
   bool m_matchParens;
+  bool m_skipTab;
   bool MatchParenthesis(int code);
-  void CloseParenthesis(wxString open, wxString close);
+  void CloseParenthesis(wxString open, wxString close, bool fromOpen);
   void OnChar(wxKeyEvent& event);
   DECLARE_EVENT_TABLE()
 };
