@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2006 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -23,6 +23,8 @@
 #include <wx/wx.h>
 #include "TextStyle.h"
 
+#include "Setup.h"
+
 class CellParser
 {
 public:
@@ -54,8 +56,10 @@ public:
   {
     return m_bottom;
   }
-  wxString GetFontName()
+  wxString GetFontName(int type = TS_DEFAULT)
   {
+    if (type == TS_TITLE || type == TS_SECTION || type == TS_TEXT)
+      return m_styles[type].font;
     return m_fontName;
   }
   wxString GetSymbolFontName();
@@ -72,10 +76,7 @@ public:
   {
     return m_haveGreekFont;
   }
-  wxString GetColor(int st)
-  {
-    return m_styles[st].color;
-  }
+  wxColour GetColor(int st);
   wxFontWeight IsBold(int st);
   int IsItalic(int st);
   int IsUnderlined(int st);
@@ -92,20 +93,43 @@ public:
   {
     return m_fontEncoding;
   }
+  bool GetChangeAsterisk()
+  {
+    return m_changeAsterisk;
+  }
+  void SetChangeAsterisk(bool changeAsterisk)
+  {
+    m_changeAsterisk = changeAsterisk;
+  }
+  int GetIndent() { return m_indent; }
+  void SetIndent(int indent) { m_indent = indent; }
+  void SetClientWidth(int width) { m_clientWidth = width; }
+  int GetClientWidth() { return m_clientWidth; }
+  int GetFontSize(int st)
+  {
+    if (st == TS_TEXT || st == TS_SECTION || st == TS_TITLE)
+      return m_styles[st].fontSize;
+    return 0;
+  }
+  void Outdated(bool outdated) { m_outdated = outdated; }
 private:
+  int m_indent;
   double m_scale;
   wxDC& m_dc;
   int m_top, m_bottom;
   wxString m_fontName;
   wxString m_greekFontName;
-#if wxUSE_UNICODE
+#if (wxUSE_UNICODE && WXM_UNICODE_GLYPHS)
   wxString m_unicodeSymbolsFont;
 #endif
   int m_greekFontAdj;
   bool m_haveGreekFont;
   bool m_forceUpdate;
+  bool m_changeAsterisk;
+  bool m_outdated;
+  int m_clientWidth;
   wxFontEncoding m_fontEncoding;
-  style m_styles[12];
+  style m_styles[STYLE_NUM];
 };
 
 #endif
