@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2006 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -27,19 +27,24 @@
 #define CONFIG_H
 
 #include "TextStyle.h"
+#include "Setup.h"
 
 enum {
-  combobox_colour,
-  combobox_styleFor,
+  color_id,
+  listbox_styleFor,
   checkbox_bold,
   checkbox_italic,
   checkbox_underlined,
   checkbox_header,
-  button_symbol,
-  checkbox_symbol,
+  button_greek,
+  checkbox_greek,
   font_family,
+  style_font_family,
   panel_size,
-  language_id
+  language_id,
+  unicode_glyphs,
+  save_id,
+  load_id
 };
 
 class ExamplePanel : public wxPanel
@@ -56,7 +61,7 @@ public:
 #endif
 
   };
-  void SetStyle(wxString fg_color, bool italic, bool bold, bool underlined, wxString font)
+  void SetStyle(wxColour fg_color, bool italic, bool bold, bool underlined, wxString font)
   {
     m_fgColor = fg_color;
     m_italic = italic;
@@ -64,21 +69,25 @@ public:
     m_underlined = underlined;
     m_font = font;
   }
+  void SetFontSize(int size) { m_size = size; }
 private:
   void OnPaint(wxPaintEvent& event);
-  wxString m_fgColor;
+  wxColour m_fgColor;
   bool m_italic, m_bold, m_underlined;
   wxString m_font;
   int m_size;
   DECLARE_EVENT_TABLE()
 };
 
+
+
 class Config: public wxDialog
 {
 public:
   Config(wxWindow* parent, int id, const wxString& title,
          const wxPoint& pos = wxDefaultPosition,
          const wxSize& size = wxDefaultSize, long style = wxDEFAULT_DIALOG_STYLE);
+  void OnChangeColor(); // called from class ColorPanel
 private:
   // begin wxGlade: Config::methods
   void set_properties();
@@ -87,11 +96,9 @@ private:
 protected:
   // begin wxGlade: Config::attributes
   wxStaticBox* sizer_11_staticbox;
-  wxStaticBox* sizer_12_staticbox;
   wxStaticBox* sizer_9_staticbox;
   wxStaticBox* sizer_6_staticbox;
   wxStaticBox* sizer_4_staticbox;
-  wxStaticText* label_1;
   wxStaticText* label_5;
   wxTextCtrl* m_maximaProgram;
   wxButton* m_mpBrowse;
@@ -105,50 +112,104 @@ protected:
   wxCheckBox* m_matchParens;
   wxCheckBox* m_showLong;
   wxCheckBox* m_showHeader;
+  wxCheckBox* m_enterEvaluates;
   wxPanel* notebook_1_pane_1;
-  wxStaticText* label_7;
-  wxSpinCtrl* m_fontSize;
   wxStaticText* label_8;
   wxButton* m_getFont;
-  wxString m_fontFamily;
+  wxButton* m_getStyleFont;
   wxFontEncoding m_fontEncoding;
-  wxComboBox* m_styleFor;
-  wxComboBox* m_styleColor;
+  wxListBox* m_styleFor;
+#ifndef __WXMSW__
+  wxPanel* m_styleColor;
+#else
+  wxButton* m_styleColor;
+#endif
   wxCheckBox* m_boldCB;
   wxCheckBox* m_italicCB;
   wxCheckBox* m_underlinedCB;
   wxCheckBox* m_fixedFontInTC;
   wxCheckBox* m_unixCopy;
+  wxCheckBox* m_changeAsterisk;
   wxPanel* notebook_1_pane_2;
   wxNotebook* notebook_1;
   wxButton* m_button1;
   wxButton* m_button2;
-  wxCheckBox* m_symbolFontOk;
-  wxButton* m_getSymbolFont;
+  wxCheckBox* m_greekFontOk;
+  wxButton* m_getGreekFont;
   wxStaticText* label_10;
-  wxSpinCtrl* m_symbolFontAdj;
-  wxString m_symbolFontName;
+  wxSpinCtrl* m_greekFontAdj;
+  wxString m_greekFontName;
+  wxButton *m_saveStyle, *m_loadStyle;
+#if (wxUSE_UNICODE && WXM_UNICODE_GLYPHS)
+  wxStaticText* m_unicodeGlyphs;
+  wxButton* m_getUnicodeFont;
+  wxString m_unicodeFont;
+#endif
+  wxStaticText* label_12;
+  wxSpinCtrl* m_defaultPort;
   ExamplePanel* label_11;
   // end wxGlade
-  style m_styleNormalText, m_styleHiddenText, m_styleMainPrompt,
-  m_styleOtherPrompt, m_styleLabel, m_styleSpecial, m_styleInput,
-  m_styleBackground, m_styleNumber, m_styleString, m_styleGreek,
-  m_styleVariable, m_styleHighlight;
+  style m_styleDefault,
+        m_styleVariable,
+        m_styleFunction,
+        m_styleNumber,
+        m_styleSpecial,
+        m_styleGreek,
+        m_styleString,
+        m_styleInput,
+        m_styleMainPrompt,
+        m_styleOtherPrompt,
+        m_styleLabel,
+        m_styleHighlight,
+        m_styleText,
+        m_styleSection,
+        m_styleTitle,
+        m_styleTextBackground,
+        m_styleBackground,
+        m_styleCellBracket,
+        m_styleActiveCellBracket,
+        m_styleCursor,
+        m_styleSelection,
+        m_styleOutdated;
   void OnOk(wxCommandEvent& event);
   void OnMpBrowse(wxCommandEvent& event);
-  void OnSymbolBrowse(wxCommandEvent& event);
+#if (wxUSE_UNICODE && WXM_UNICODE_GLYPHS)
+  void OnChangeUnicodeFont(wxCommandEvent& event);
+#endif
+#if defined __WXMSW__
+  void OnColorButton(wxCommandEvent& event);
+#endif
+  void OnGreekBrowse(wxCommandEvent& event);
   void OnChangeStyle(wxCommandEvent& event);
-  void OnChangeColor(wxCommandEvent& event);
   void OnChangeWarning(wxCommandEvent& event);
   void OnCheckbox(wxCommandEvent& event);
-  void OnCheckSymbol(wxCommandEvent& event);
-  void ReadStyles();
-  void WriteStyles();
+  void OnCheckGreek(wxCommandEvent& event);
+  void ReadStyles(wxString file = wxEmptyString);
+  void WriteStyles(wxString file = wxEmptyString);
   void SetupFontList();
   void UpdateExample();
   void OnChangeFontFamily(wxCommandEvent& event);
+  void LoadSave(wxCommandEvent& event);
+  int m_fontSize;
   style* GetStylePointer();
   DECLARE_EVENT_TABLE()
 };
+#ifndef __WXMSW__
+class ColorPanel : public wxPanel
+{
+public:
+  ColorPanel(Config * conf, wxWindow *parent, int id, wxPoint pos, wxSize size, long style) : wxPanel(parent, id, pos, size, style)
+  {
+     config = conf;
+     SetBackgroundColour(wxColour(0,0,0));
+  };
+  void OnClick(wxMouseEvent &event) {
+      config->OnChangeColor();
+  }
+private:
+  Config * config;
+  DECLARE_EVENT_TABLE()
+};
+#endif // __WXMSW__
 
 #endif // CONFIG_H
