@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2006 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2006-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -17,17 +17,18 @@
 ///  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ///
 
+#include <wx/clipbrd.h>
+
 #include "EditorCell.h"
 #include "wxMaxima.h"
 #include "wxMaximaFrame.h"
 
-#include <wx/clipbrd.h>
-
 EditorCell::EditorCell() : MathCell()
 {
   m_text = wxEmptyString;
   m_fontSize = -1;
   m_positionOfCaret = 0;
+  m_caretColumn = -1; // used when moving up/down between lines
   m_selectionStart = -1;
   m_selectionEnd = -1;
   m_isActive = false;
@@ -39,6 +40,11 @@ EditorCell::EditorCell() : MathCell()
   m_fontWeight = wxFONTWEIGHT_NORMAL;
   m_fontStyle = wxNORMAL;
   m_fontEncoding = wxFONTENCODING_DEFAULT;
+  m_oldStart = -1;
+  m_oldEnd = -1;
+  m_saveValue = false;
+  m_containsChanges = false;
+  m_containsChangesCheck = false;
 }
 
 EditorCell::~EditorCell()
@@ -51,6 +57,7 @@ MathCell *EditorCell::Copy(bool all)
 {
   EditorCell *tmp = new EditorCell();
   tmp->SetValue(m_text);
+  tmp->m_containsChanges = m_containsChanges;
   CopyData(this, tmp);
   if (all && m_next != NULL)
     tmp->AppendCell(m_next->Copy(all));
@@ -65,9 +72,55 @@ void EditorCell::Destroy()
 wxString EditorCell::ToString(bool all)
 {
   wxString text = m_text;
+
+  if (m_selectionStart > -1)
+  {
+    long start = MIN(m_selectionStart, m_selectionEnd);
+    long end = MAX(m_selectionStart, m_selectionEnd) - 1;
+    text = m_text.SubString(start, end);
+  }
+
   return text + MathCell::ToString(all);
 }
 
+wxString EditorCell::ToTeX(bool all)
+{
+  wxString text = m_text;
+  return text + MathCell::ToTeX(all);
+}
+
+wxString EditorCell::ToXML(bool all)
+{
+  wxString xmlstring = m_text;
+  // convert it, so that the XML parser doesn't fail
+  xmlstring.Replace(wxT("&"),  wxT("&amp;"));
+  xmlstring.Replace(wxT("<"),  wxT("&lt;"));
+  xmlstring.Replace(wxT(">"),  wxT("&gt;"));
+  xmlstring.Replace(wxT("'"),  wxT("&apos;"));
+  xmlstring.Replace(wxT("\""), wxT("&quot;"));
+  xmlstring.Replace(wxT("\n"), wxT("</line>\n<line>"));
+  xmlstring = wxT("<line>") + xmlstring + wxT("</line>\n");
+  wxString head = wxT("<editor");
+  switch (m_type) {
+    case MC_TYPE_TEXT:
+      head += wxT(" type=\"text\"");
+      break;
+    case MC_TYPE_TITLE:
+      head += wxT(" type=\"title\"");
+      break;
+    case MC_TYPE_SECTION:
+      head += wxT(" type=\"section\"");
+      break;
+    case MC_TYPE_INPUT:
+    default:
+      head += wxT(" type=\"input\"");
+      break;
+  }
+  head += wxT(">\n");
+
+  return head + xmlstring + wxT("</editor>\n") + MathCell::ToXML(all);
+}
+
 void EditorCell::RecalculateWidths(CellParser& parser, int fontsize, bool all)
 {
   m_isDirty = false;
@@ -122,25 +175,109 @@ void EditorCell::RecalculateSize(CellParser& parser, int fontsize, bool all)
   MathCell::RecalculateSize(parser, fontsize, all);
 }
 
-void EditorCell::Draw(CellParser& parser, wxPoint point, int fontsize, bool all)
+///////////////////////////
+// EditorCell::Draw
+// Draws the editor cell in the following order:
+// 1. draw selection (wxCOPY), TS_SELECTION color
+// 2. mark matching parenthesis (wxCOPY), TS_SELECTION color
+// 3. draw text (wxCOPY)
+// 4. draw caret (wxCOPY), TS_CURSOR color
+////////////////////////////
+void EditorCell::Draw(CellParser& parser, wxPoint point1, int fontsize, bool all)
 {
   double scale = parser.GetScale();
   wxDC& dc = parser.GetDC();
+  wxPoint point(point1);
 
   if (m_width == -1 || m_height == -1)
     RecalculateWidths(parser, fontsize, false);
 
   if (DrawThisCell(parser, point) && !m_isHidden)
   {
+    dc.SetLogicalFunction(wxCOPY); // opaque (for everything except the caret)
+
+    if (m_isActive) // draw selection or matching parens
+    {
+      //
+      // Mark selection
+      //
+      if (m_selectionStart > -1)
+      {
+#if defined(__WXMAC__)
+        wxRect rect = GetRect(); // rectangle representing the cell
+        dc.SetPen(wxNullPen); // no border on rectangles
+#else
+        dc.SetPen(*(wxThePenList->FindOrCreatePen(parser.GetColor(TS_SELECTION), 1, 1)) );
+// window linux, set a pen
+#endif
+        dc.SetBrush( *(wxTheBrushList->FindOrCreateBrush(parser.GetColor(TS_SELECTION))) ); //highlight c.
+
+        wxPoint point, point1;
+        long start = MIN(m_selectionStart, m_selectionEnd);
+        long end = MAX(m_selectionStart, m_selectionEnd);
+        long pos1 = start, pos2 = start;
+
+        while (pos1 < end) // go through selection, draw a rect for each line of selection
+        {
+          while (pos1 < end && m_text.GetChar(pos1) != '\n')
+            pos1++;
+
+          point = PositionToPoint(parser, pos2);  // left  point
+          point1 = PositionToPoint(parser, pos1); // right point
+          long selectionWidth = point1.x - point.x;
+#if defined(__WXMAC__)
+          if (pos1 != end) // we have a \n, draw selection to the right border (mac behaviour)
+            selectionWidth = rect.GetRight() - point.x - SCALE_PX(2,scale);
+#endif
+          dc.DrawRectangle(point.x + SCALE_PX(2, scale), // draw the rectangle
+                           point.y + SCALE_PX(2, scale) - m_center,
+                           selectionWidth,
+                           m_charHeight);
+          pos1++;
+          pos2 = pos1;
+        }
+      } // if (m_selectionStart > -1)
+
+      //
+      // Matching parens - draw only if we dont have selection
+      //
+      else if (m_paren1 != -1 && m_paren2 != -1)
+      {
+#if defined(__WXMAC__)
+        wxRect rect = GetRect(); // rectangle representing the cell
+        dc.SetPen(wxNullPen); // no border on rectangles
+#else
+        dc.SetPen(*(wxThePenList->FindOrCreatePen(parser.GetColor(TS_SELECTION), 1, 1))
+); // window linux, set a pen
+#endif
+        dc.SetBrush( *(wxTheBrushList->FindOrCreateBrush(parser.GetColor(TS_SELECTION))) ); //highlight c.
+
+        wxPoint point = PositionToPoint(parser, m_paren1);
+        int width, height;
+        dc.GetTextExtent(m_text.GetChar(m_paren1), &width, &height);
+        dc.DrawRectangle(point.x + SCALE_PX(2, scale) + 1,
+                         point.y  + SCALE_PX(2, scale) - m_center + 1,
+                         width - 1, height - 1);
+        point = PositionToPoint(parser, m_paren2);
+        dc.GetTextExtent(m_text.GetChar(m_paren1), &width, &height);
+        dc.DrawRectangle(point.x + SCALE_PX(2, scale) + 1,
+                         point.y  + SCALE_PX(2, scale) - m_center + 1,
+                         width - 1, height - 1);
+      } // else if (m_paren1 != -1 && m_paren2 != -1)
+    } // if (m_isActive)
+
+    //
+    // Draw the text
+    //
     SetForeground(parser);
     SetPen(parser);
     SetFont(parser, fontsize);
 
     unsigned int newLinePos = 0, prevNewLinePos = 0, numberOfLines = 0;
-
-    //
-    // Draw the text
-    //
+#if defined __WXMSW__ || wxUSE_UNICODE
+    if (parser.GetChangeAsterisk())  // replace "*" with centerdot for the time of drawing
+      m_text.Replace(wxT("*"), wxT("\xB7"));
+#endif
     while (newLinePos < m_text.Length())
     {
       while (newLinePos < m_text.Length())
@@ -158,86 +295,43 @@ void EditorCell::Draw(CellParser& parser, wxPoint point, int fontsize, bool all)
       prevNewLinePos = newLinePos;
       numberOfLines++;
     }
-
-    if (m_isActive)
+#if defined __WXMSW__ || wxUSE_UNICODE
+    if (parser.GetChangeAsterisk()) // replace centerdot with "*"
+      m_text.Replace(wxT("\xB7"), wxT("*"));
+#endif
+    //
+    // Draw the caret
+    //
+    if (m_displayCaret && m_hasFocus && m_isActive)
     {
-      //
-      // Draw the caret
-      //
-      if (m_displayCaret && m_hasFocus)
-      {
-        int caretInLine = 0;
-        int caretInColumn = 0;
+      int caretInLine = 0;
+      int caretInColumn = 0;
 
-        PositionToXY(m_positionOfCaret, &caretInColumn, &caretInLine);
+      PositionToXY(m_positionOfCaret, &caretInColumn, &caretInLine);
 
-        wxString line = GetLineString(caretInLine, 0, caretInColumn);
-        int lineWidth, lineHeight;
-        dc.GetTextExtent(line, &lineWidth, &lineHeight);
+      wxString line = GetLineString(caretInLine, 0, caretInColumn);
+      int lineWidth, lineHeight;
+      dc.GetTextExtent(line, &lineWidth, &lineHeight);
 
-
-        dc.DrawLine(point.x + SCALE_PX(2, scale) + lineWidth,
-                    point.y + SCALE_PX(2, scale) - m_center + caretInLine * m_charHeight,
-                    point.x + SCALE_PX(2, scale) + lineWidth,
-                    point.y + SCALE_PX(2, scale) - m_center + (caretInLine + 1) * m_charHeight);
-      }
-
-      //
-      // Mark selection
-      //
-      if (m_selectionStart > -1)
-      {
-        dc.SetLogicalFunction(wxAND);
-        dc.SetBrush(*wxLIGHT_GREY_BRUSH);
-        dc.SetPen(*wxLIGHT_GREY_PEN);
-        wxPoint point;
-        long start = MIN(m_selectionStart, m_selectionEnd);
-        long end = MAX(m_selectionStart, m_selectionEnd);
-        long pos1 = start, pos2 = start;
-        wxPoint point1;
-
-        while (pos1 < end)
-        {
-          while (pos1 < end && m_text.GetChar(pos1) != '\n')
-            pos1++;
-          point = PositionToPoint(parser, pos2);
-          point1 = PositionToPoint(parser, pos1);
-          dc.DrawRectangle(point.x + SCALE_PX(2, scale),
-                           point.y  + SCALE_PX(2, scale) - m_center,
-                           point1.x - point.x,
-                           m_charHeight);
-          pos1++;
-          pos2 = pos1;
-        }
-
-        dc.SetLogicalFunction(wxCOPY);
-      }
-
-      //
-      // Matching parens
-      //
-      else if (m_paren1 != -1 && m_paren2 != -1)
-      {
-        dc.SetLogicalFunction(wxAND);
-        dc.SetBrush(*wxLIGHT_GREY_BRUSH);
-        dc.SetPen(*wxLIGHT_GREY_PEN);
-        wxPoint point = PositionToPoint(parser, m_paren1);
-        int width, height;
-        dc.GetTextExtent(m_text.GetChar(m_paren1), &width, &height);
-        dc.DrawRectangle(point.x + SCALE_PX(2, scale),
-                         point.y  + SCALE_PX(2, scale) - m_center,
-                         width, height);
-        point = PositionToPoint(parser, m_paren2);
-        dc.GetTextExtent(m_text.GetChar(m_paren1), &width, &height);
-        dc.DrawRectangle(point.x + SCALE_PX(2, scale),
-                         point.y  + SCALE_PX(2, scale) - m_center,
-                         width, height);
-        dc.SetLogicalFunction(wxCOPY);
-      }
+      dc.SetPen(*(wxThePenList->FindOrCreatePen(parser.GetColor(TS_CURSOR), 1, wxSOLID))); //TODO is there more efficient way to do this?
+#if defined(__WXMAC__)
+      // draw 1 pixel shorter caret than on windows
+      dc.DrawLine(point.x + SCALE_PX(2, scale) + lineWidth,
+                  point.y + SCALE_PX(2, scale) - m_center + caretInLine * m_charHeight,
+                  point.x + SCALE_PX(2, scale) + lineWidth,
+                  point.y + SCALE_PX(1, scale) - m_center + (caretInLine + 1) * m_charHeight);
+#else
+      dc.DrawLine(point.x + SCALE_PX(2, scale) + lineWidth,
+                  point.y + SCALE_PX(2, scale) - m_center + caretInLine * m_charHeight,
+                  point.x + SCALE_PX(2, scale) + lineWidth,
+                  point.y + SCALE_PX(2, scale) - m_center + (caretInLine + 1) * m_charHeight);
+#endif
     }
+
     UnsetPen(parser);
-  }
-  MathCell::Draw(parser, point, fontsize, all);
+
+  } // if (DrawThisCell(parser, point) && !m_isHidden)
+  MathCell::Draw(parser, point1, fontsize, all);
 }
 
 void EditorCell::SetFont(CellParser& parser, int fontsize)
@@ -245,32 +339,20 @@ void EditorCell::SetFont(CellParser& parser, int fontsize)
   wxDC& dc = parser.GetDC();
   double scale = parser.GetScale();
 
-  m_fontSize = fontsize;
-  int fontsize1 = (int) (((double)fontsize) * scale + 0.5);
+  int fontsize1 = parser.GetFontSize(m_textStyle);
+  if (fontsize1 == 0)
+    fontsize1 = fontsize;
+  m_fontSize = fontsize1;
+
+  fontsize1 = (int) (((double)fontsize1) * scale + 0.5);
   fontsize1 = MAX(fontsize1, 1);
 
-  m_fontName = parser.GetFontName();
+  m_fontName = parser.GetFontName(m_textStyle);
+  m_fontEncoding = parser.GetFontEncoding();
+  m_fontStyle = parser.IsItalic(m_textStyle);
+  m_fontWeight = parser.IsBold(m_textStyle);
+  m_underlined = parser.IsUnderlined(m_textStyle);
   m_fontEncoding = parser.GetFontEncoding();
-
-  switch(m_type)
-  {
-  case MC_TYPE_TITLE:
-    fontsize1 += 4;
-    m_fontStyle = wxFONTSTYLE_SLANT;
-  case MC_TYPE_SECTION:
-    fontsize1 += 4;
-    m_fontWeight = wxFONTWEIGHT_BOLD;
-    m_underlined = true;
-  case MC_TYPE_COMMENT:
-    m_fontEncoding = parser.GetFontEncoding();
-    break;
-  default:
-    m_fontStyle = parser.IsItalic(TS_INPUT);
-    m_fontWeight = parser.IsBold(TS_INPUT);
-    m_underlined = parser.IsUnderlined(TS_INPUT);
-    m_fontEncoding = parser.GetFontEncoding();
-    break;
-  }
 
   dc.SetFont(wxFont(fontsize1, wxMODERN,
                     m_fontStyle,
@@ -283,30 +365,64 @@ void EditorCell::SetFont(CellParser& parser, int fontsize)
 void EditorCell::SetForeground(CellParser& parser)
 {
   wxDC& dc = parser.GetDC();
-
-  switch (m_type)
-  {
-  case MC_TYPE_COMMENT:
-  case MC_TYPE_SECTION:
-  case MC_TYPE_TITLE:
-    dc.SetTextForeground(wxTheColourDatabase->Find(parser.GetColor(TS_NORMAL_TEXT)));
-    break;
-  default:
-    dc.SetTextForeground(wxTheColourDatabase->Find(parser.GetColor(TS_INPUT)));
-    break;
-  }
+  dc.SetTextForeground(parser.GetColor(m_textStyle));
 }
 
+#ifndef WX_USE_UNICODE
+
+int ChangeNumpadToChar(int c)
+{
+  switch (c) {
+    case WXK_NUMPAD0:
+      return '0';
+      break;
+    case WXK_NUMPAD1:
+      return '1';
+      break;
+    case WXK_NUMPAD2:
+      return '2';
+      break;
+    case WXK_NUMPAD3:
+      return '3';
+      break;
+    case WXK_NUMPAD4:
+      return '4';
+      break;
+    case WXK_NUMPAD5:
+      return '5';
+      break;
+    case WXK_NUMPAD6:
+      return '6';
+      break;
+    case WXK_NUMPAD7:
+      return '7';
+      break;
+    case WXK_NUMPAD8:
+      return '8';
+      break;
+    case WXK_NUMPAD9:
+      return '9';
+      break;
+    case WXK_NUMPAD_DECIMAL:
+      return '.';
+      break;
+  }
+  return c;
+}
+
+#endif
+
 void EditorCell::ProcessEvent(wxKeyEvent &event)
 {
+  if ((event.GetKeyCode() != WXK_DOWN) &&
+      (event.GetKeyCode() != WXK_PAGEDOWN) &&
+      (event.GetKeyCode() != WXK_PAGEUP) &&
+      (event.GetKeyCode() != WXK_UP))
+      m_caretColumn = -1; // make caretColumn invalid
+
   switch (event.GetKeyCode())
   {
-  case WXK_ESCAPE:
-    {
-      wxCommandEvent ev(wxEVT_COMMAND_MENU_SELECTED, deactivate_cell_cancel);
-      (wxGetApp().GetTopWindow())->ProcessEvent(ev);
-    }
-    break;
+
   case WXK_LEFT:
     if (event.ShiftDown())
     {
@@ -315,11 +431,15 @@ void EditorCell::ProcessEvent(wxKeyEvent &event)
     }
     else
       m_selectionEnd = m_selectionStart = -1;
+
     if (m_positionOfCaret > 0)
       m_positionOfCaret--;
+
     if (event.ShiftDown())
       m_selectionEnd = m_positionOfCaret;
+
     break;
+
   case WXK_RIGHT:
     if (event.ShiftDown())
     {
@@ -328,11 +448,15 @@ void EditorCell::ProcessEvent(wxKeyEvent &event)
     }
     else
       m_selectionEnd = m_selectionStart = -1;
+
     if (m_positionOfCaret < m_text.Length())
       m_positionOfCaret++;
+
     if (event.ShiftDown())
       m_selectionEnd = m_positionOfCaret;
+
     break;
+
   case WXK_PAGEDOWN:
   case WXK_DOWN:
     {
@@ -343,17 +467,26 @@ void EditorCell::ProcessEvent(wxKeyEvent &event)
       }
       else
         m_selectionEnd = m_selectionStart = -1;
+
       int column, line;
-      PositionToXY(m_positionOfCaret, &column, &line);
+      PositionToXY(m_positionOfCaret, &column, &line); // get current line
+      if (m_caretColumn > -1)
+        column = m_caretColumn;
+      else
+        m_caretColumn = column;
 
-      line = line < m_numberOfLines-1 ? line + 1 : line;
-
-      m_positionOfCaret = XYToPosition(column, line);
+      if (line < m_numberOfLines-1) // can we go down ?
+        m_positionOfCaret = XYToPosition(column, line + 1);
+      else { // we can't go down. move caret to the end
+        m_positionOfCaret = m_text.Length();
+        m_caretColumn = -1; // make caretColumn invalid
+      }
 
       if (event.ShiftDown())
         m_selectionEnd = m_positionOfCaret;
     }
     break;
+
   case WXK_PAGEUP:
   case WXK_UP:
     {
@@ -364,23 +497,44 @@ void EditorCell::ProcessEvent(wxKeyEvent &event)
       }
       else
         m_selectionEnd = m_selectionStart = -1;
+
       int column, line;
-      PositionToXY(m_positionOfCaret, &column, &line);
+      PositionToXY(m_positionOfCaret, &column, &line); // get current line
+      if (m_caretColumn > -1)
+        column = m_caretColumn;
+      else
+        m_caretColumn = column;
 
-      line = line > 0 ? line - 1 : 0;
+      if (line > 0) // can we go up?
+        m_positionOfCaret = XYToPosition(column, line - 1);
+      else { // we can't move up, move to the beginning
+        m_positionOfCaret = 0;
+        m_caretColumn = -1; // make caretColumn invalid
+      }
 
-      m_positionOfCaret = XYToPosition(column, line);
       if (event.ShiftDown())
         m_selectionEnd = m_positionOfCaret;
     }
     break;
+
   case WXK_RETURN:
+    if (m_selectionStart != -1) // we have a selection, delete it, then proceed
+    {
+      long start = MIN(m_selectionEnd, m_selectionStart);
+      long end = MAX(m_selectionEnd, m_selectionStart);
+      m_text = m_text.SubString(0, start - 1) +
+               m_text.SubString(end, m_text.Length());
+      m_positionOfCaret = start;
+      m_selectionEnd = m_selectionStart = -1;
+    }
     m_text = m_text.SubString(0, m_positionOfCaret - 1) +
              wxT("\n") +
              m_text.SubString(m_positionOfCaret, m_text.Length());
     m_positionOfCaret++;
     m_isDirty = true;
+    m_containsChanges = true;
     break;
+
   case WXK_END:
     if (event.ShiftDown())
     {
@@ -389,6 +543,7 @@ void EditorCell::ProcessEvent(wxKeyEvent &event)
     }
     else
       m_selectionEnd = m_selectionStart = -1;
+
     if (event.ControlDown())
       m_positionOfCaret = m_text.Length();
     else
@@ -397,9 +552,11 @@ void EditorCell::ProcessEvent(wxKeyEvent &event)
              m_text.GetChar(m_positionOfCaret) != '\n')
         m_positionOfCaret++;
     }
+
     if (event.ShiftDown())
       m_selectionEnd = m_positionOfCaret;
     break;
+
   case WXK_HOME:
     {
       if (event.ShiftDown())
@@ -409,6 +566,7 @@ void EditorCell::ProcessEvent(wxKeyEvent &event)
       }
       else
         m_selectionEnd = m_selectionStart = -1;
+
       if (event.ControlDown())
         m_positionOfCaret = 0;
       else
@@ -417,31 +575,44 @@ void EditorCell::ProcessEvent(wxKeyEvent &event)
         PositionToXY(m_positionOfCaret, &col, &lin);
         m_positionOfCaret = XYToPosition(0, lin);
       }
+
       if (event.ShiftDown())
         m_selectionEnd = m_positionOfCaret;
     }
     break;
+
   case WXK_DELETE:
-    m_isDirty = true;
-    if (m_positionOfCaret < m_text.Length())
+    if (m_selectionStart == -1)
     {
-      if (m_selectionStart == -1)
+      if (m_positionOfCaret < m_text.Length())
+      {
+        m_isDirty = true;
+        m_containsChanges = true;
         m_text = m_text.SubString(0, m_positionOfCaret - 1) +
                  m_text.SubString(m_positionOfCaret + 1, m_text.Length());
-      else
-      {
-        long start = MIN(m_selectionEnd, m_selectionStart);
-        long end = MAX(m_selectionEnd, m_selectionStart);
-        m_text = m_text.SubString(0, start - 1) +
-                 m_text.SubString(end, m_text.Length());
-        m_positionOfCaret = start;
-        m_selectionEnd = m_selectionStart = -1;
       }
     }
+    else
+    {
+      m_isDirty = true;
+      m_containsChanges = true;
+      SaveValue();
+      m_saveValue = true;
+      long start = MIN(m_selectionEnd, m_selectionStart);
+      long end = MAX(m_selectionEnd, m_selectionStart);
+      m_text = m_text.SubString(0, start - 1) +
+               m_text.SubString(end, m_text.Length());
+      m_positionOfCaret = start;
+      m_selectionEnd = m_selectionStart = -1;
+    }
     break;
+
   case WXK_BACK:
-    m_isDirty = true;
     if (m_selectionStart > -1) {
+      SaveValue();
+      m_saveValue = true;
+      m_containsChanges = true;
+      m_isDirty = true;
       long start = MIN(m_selectionEnd, m_selectionStart);
       long end = MAX(m_selectionEnd, m_selectionStart);
       m_text = m_text.SubString(0, start - 1) +
@@ -452,13 +623,17 @@ void EditorCell::ProcessEvent(wxKeyEvent &event)
     }
     else if (m_positionOfCaret > 0)
     {
+      m_containsChanges = true;
+      m_isDirty = true;
       m_text = m_text.SubString(0, m_positionOfCaret - 2) +
                m_text.SubString(m_positionOfCaret, m_text.Length());
       m_positionOfCaret--;
     }
     break;
+
   case WXK_TAB:
     m_isDirty = true;
+    m_containsChanges = true;
     {
       if (m_selectionStart > -1) {
         long start = MIN(m_selectionEnd, m_selectionStart);
@@ -469,6 +644,7 @@ void EditorCell::ProcessEvent(wxKeyEvent &event)
         m_selectionEnd = m_selectionStart = -1;
         break;
       }
+
       int col, line;
       PositionToXY(m_positionOfCaret, &col, &line);
       wxString ins;
@@ -483,54 +659,149 @@ void EditorCell::ProcessEvent(wxKeyEvent &event)
       m_positionOfCaret += ins.Length();
     }
     break;
+
+  case WXK_SPACE:
+    if (event.ShiftDown())
+      m_text = m_text.SubString(0, m_positionOfCaret - 1) + wxT("*") + // wxT("\x00B7")
+               m_text.SubString(m_positionOfCaret, m_text.Length());
+    else
+      m_text = m_text.SubString(0, m_positionOfCaret - 1) + wxT(" ") +
+               m_text.SubString(m_positionOfCaret, m_text.Length());
+    m_isDirty = true;
+    m_containsChanges = true;
+    m_positionOfCaret++;
+    break;
+
   default:
     if (event.ControlDown())
       break;
+
     m_isDirty = true;
+    m_containsChanges = true;
+    bool insertLetter = true;
+
+    if (m_saveValue) {
+      SaveValue();
+      m_saveValue = false;
+    }
+    // if we have a selection either put parens around it (and don't write the letter afterwards)
+    // od delete selection and write letter (insertLetter = true).
     if (m_selectionStart > -1) {
       long start = MIN(m_selectionEnd, m_selectionStart);
       long end = MAX(m_selectionEnd, m_selectionStart);
-      m_text = m_text.SubString(0, start - 1) +
-               m_text.SubString(end, m_text.Length());
-      m_positionOfCaret = start;
-      m_selectionEnd = m_selectionStart = -1;
-    }
-    m_text = m_text.SubString(0, m_positionOfCaret - 1) +
 #if wxUSE_UNICODE
-             event.GetUnicodeKey() +
+      switch (event.GetUnicodeKey())
 #else
-             wxString::Format(wxT("%c"), event.GetKeyCode()) +
-#endif
-             m_text.SubString(m_positionOfCaret, m_text.Length());
-    m_positionOfCaret++;
-    if (m_matchParens)
-    {
       switch (event.GetKeyCode())
+#endif
       {
       case '(':
-        m_text = m_text.SubString(0, m_positionOfCaret - 1) +
-                 wxT(")") +
-                 m_text.SubString(m_positionOfCaret, m_text.Length());
-        break;
-      case '[':
-        m_text = m_text.SubString(0, m_positionOfCaret - 1) +
-                 wxT("]") +
-                 m_text.SubString(m_positionOfCaret, m_text.Length());
+        m_text = m_text.SubString(0, start - 1) +   wxT("(") +
+                 m_text.SubString(start, end - 1) + wxT(")") +
+                 m_text.SubString(end, m_text.Length());
+        m_positionOfCaret = start;  insertLetter = false;
         break;
       case '{':
-        m_text = m_text.SubString(0, m_positionOfCaret - 1) +
-                 wxT("}") +
-                 m_text.SubString(m_positionOfCaret, m_text.Length());
+        m_text = m_text.SubString(0, start - 1) +   wxT("{") +
+                 m_text.SubString(start, end - 1) + wxT("}") +
+                 m_text.SubString(end, m_text.Length());
+        m_positionOfCaret = start;  insertLetter = false;
+        break;
+      case '[':
+        m_text = m_text.SubString(0, start - 1) +   wxT("[") +
+                 m_text.SubString(start, end - 1) + wxT("]") +
+                 m_text.SubString(end, m_text.Length());
+        m_positionOfCaret = start;  insertLetter = false;
+        break;
+      case ')':
+        m_text = m_text.SubString(0, start - 1) +   wxT("(") +
+                 m_text.SubString(start, end - 1) + wxT(")") +
+                 m_text.SubString(end, m_text.Length());
+        m_positionOfCaret = end + 2; insertLetter = false;
+        break;
+      case '}':
+        m_text = m_text.SubString(0, start - 1) +   wxT("{") +
+                 m_text.SubString(start, end - 1) + wxT("}") +
+                 m_text.SubString(end, m_text.Length());
+        m_positionOfCaret = end + 2; insertLetter = false;
+        break;
+      case ']':
+        m_text = m_text.SubString(0, start - 1) +   wxT("[") +
+                 m_text.SubString(start, end - 1) + wxT("]") +
+                 m_text.SubString(end, m_text.Length());
+        m_positionOfCaret = end + 2; insertLetter = false;
+        break;
+      default: // delete selection
+        m_text = m_text.SubString(0, start - 1) +
+                 m_text.SubString(end, m_text.Length());
+        m_positionOfCaret = start;
         break;
       }
-    }
+      m_selectionEnd = m_selectionStart = -1; // reset selection
+    } // end if (m_selectionStart > -1)
+
+// insert letter if we didnt insert brackets around selection
+  if (insertLetter) {
+      m_text = m_text.SubString(0, m_positionOfCaret - 1) +
+#if wxUSE_UNICODE
+               event.GetUnicodeKey() +
+#else
+               wxString::Format(wxT("%c"), ChangeNumpadToChar(event.GetKeyCode())) +
+#endif
+               m_text.SubString(m_positionOfCaret, m_text.Length());
+
+      m_positionOfCaret++;
+
+      if (m_matchParens)
+      {
+#if wxUSE_UNICODE
+        switch (event.GetUnicodeKey())
+#else
+        switch (event.GetKeyCode())
+#endif
+        {
+        case '(':
+          m_text = m_text.SubString(0, m_positionOfCaret - 1) +
+                   wxT(")") +
+                   m_text.SubString(m_positionOfCaret, m_text.Length());
+          break;
+        case '[':
+          m_text = m_text.SubString(0, m_positionOfCaret - 1) +
+                   wxT("]") +
+                   m_text.SubString(m_positionOfCaret, m_text.Length());
+          break;
+        case '{':
+          m_text = m_text.SubString(0, m_positionOfCaret - 1) +
+                   wxT("}") +
+                   m_text.SubString(m_positionOfCaret, m_text.Length());
+          break;
+        case ')': // jump over ')'
+          if (m_text.GetChar(m_positionOfCaret) == ')')
+            m_text = m_text.SubString(0, m_positionOfCaret - 2) +
+                       m_text.SubString(m_positionOfCaret, m_text.Length());
+          break;
+        case ']': // jump over ']'
+          if (m_text.GetChar(m_positionOfCaret) == ']')
+            m_text = m_text.SubString(0, m_positionOfCaret - 2) +
+                       m_text.SubString(m_positionOfCaret, m_text.Length());
+          break;
+        case '}': // jump over '}'
+          if (m_text.GetChar(m_positionOfCaret) == '}')
+            m_text = m_text.SubString(0, m_positionOfCaret - 2) +
+                       m_text.SubString(m_positionOfCaret, m_text.Length());
+          break;
+        }
+      }
+    } // end if (insertLetter)
     break;
-  }
+  } // end switch (event.GetKeyCode())
 
   if (m_type == MC_TYPE_INPUT)
     FindMatchingParens();
+
   if (m_isDirty)
-    m_width = m_height = m_maxDrop = m_center = -1;
+    m_width = m_maxDrop = -1;
+
   m_displayCaret = true;
 }
 
@@ -583,16 +854,19 @@ void EditorCell::FindMatchingParens()
 
   m_paren1 = m_paren2 + dir;
   int depth = 1;
+
   while (m_paren1 >= 0 && m_paren1 < (int)m_text.Length())
   {
     if (m_text.GetChar(m_paren1) == second)
       depth--;
     else if (m_text.GetChar(m_paren1) == first)
       depth++;
+
     if (depth == 0)
       break;
     m_paren1 += dir;
   }
+
   if (m_paren1 < 0 || m_paren1 >= (int)m_text.Length())
     m_paren1 = m_paren2 = -1;
 }
@@ -600,19 +874,26 @@ void EditorCell::FindMatchingParens()
 bool EditorCell::ActivateCell()
 {
   m_isActive = !m_isActive;
+  if (m_isActive)
+    SaveValue();
   m_displayCaret = true;
   m_hasFocus = true;
 
   m_selectionEnd = m_selectionStart = -1;
+  m_paren1 = m_paren2 = -1;
 
   return true;
 }
 
-void EditorCell::AddEnding()
+bool EditorCell::AddEnding()
 {
   wxString text = m_text.Trim();
-  if (text.Right(1) != wxT(";") && text.Right(1) != wxT("$"))
+  if (text.Right(1) != wxT(";") && text.Right(1) != wxT("$")) {
     m_text += wxT(";");
+    m_width = -1;
+    return true;
+  }
+  return false;
 }
 
 //
@@ -696,11 +977,6 @@ void EditorCell::SelectPointText(wxDC& dc, wxPoint& point)
   wxString s;
   int fontsize1 = m_fontSize;
 
-  if (m_type == MC_TYPE_TITLE)
-    fontsize1 += 8;
-  else if (m_type == MC_TYPE_SECTION)
-    fontsize1 += 4;
-
   dc.SetFont(wxFont(fontsize1, wxMODERN,
                     m_fontStyle,
                     m_fontWeight,
@@ -733,6 +1009,7 @@ void EditorCell::SelectPointText(wxDC& dc, wxPoint& point)
   m_positionOfCaret = MIN(m_positionOfCaret, m_text.Length());
 
   m_displayCaret = true;
+  m_caretColumn = -1;
   if (GetType() == MC_TYPE_INPUT)
     FindMatchingParens();
 }
@@ -745,6 +1022,7 @@ void EditorCell::SelectRectText(wxDC &dc, wxPoint& one, wxPoint& two)
   m_selectionEnd = m_positionOfCaret;
   m_selectionStart = start;
   m_paren2 = m_paren1 = -1;
+  m_caretColumn = -1;
   if (m_selectionStart == m_selectionEnd)
   {
     m_selectionStart = -1;
@@ -752,6 +1030,95 @@ void EditorCell::SelectRectText(wxDC &dc, wxPoint& one, wxPoint& two)
   }
 }
 
+// IsPointInSelection
+// Return true if coordinates "point" fall into selection
+// If they don't or there is no selection it returns false
+bool EditorCell::IsPointInSelection(wxDC& dc, wxPoint point)
+{
+  if ((m_selectionStart == -1) || (m_selectionEnd == -1) || (m_isActive == false))
+    return false;
+
+  wxRect rect = GetRect();
+  if (!rect.Contains(point))
+    return false;
+
+  wxString s;
+  int fontsize1 = m_fontSize;
+
+  dc.SetFont(wxFont(fontsize1, wxMODERN,
+                    m_fontStyle,
+                    m_fontWeight,
+                    m_underlined,
+                    m_fontName,
+                    m_fontEncoding));
+  wxPoint translate(point);
+  translate.x -= m_currentPoint.x - 2;
+  translate.y -= m_currentPoint.y - 2 - m_center;
+  int lin = translate.y / m_charHeight;
+  int width, height;
+  int lineStart = XYToPosition(0, lin);
+  int positionOfCaret = lineStart;
+  while (m_text.GetChar(positionOfCaret) != '\n' && positionOfCaret < m_text.Length())
+  {
+    s = m_text.SubString(lineStart, positionOfCaret);
+    dc.GetTextExtent(m_text.SubString(lineStart, positionOfCaret),
+                                      &width, &height);
+    if (width > translate.x)
+      break;
+    positionOfCaret++;
+  }
+  positionOfCaret = MIN(positionOfCaret, m_text.Length());
+
+  if ((m_selectionStart >= positionOfCaret) || (m_selectionEnd <= positionOfCaret))
+    return false;
+
+  return true;
+}
+/***
+ * SelectWordUnderCaret
+ * - called from MathCtrl::OnDoubleClick and wxMaxima::HelpMenu
+ * Selects word under cursor (aA-zZ, 0-9 counts) or
+ * the inside of brackets using m_paren1 and m_paren2 if available.
+ * Returns the selected string if selected a word succesfully - used for F1 help.
+ */
+wxString EditorCell::SelectWordUnderCaret()
+{
+  if ((m_paren1 != -1) && (m_paren2 != -1)) {
+    m_selectionStart = MIN(m_paren1,m_paren2) + 1;
+    m_selectionEnd = MAX(m_paren1, m_paren2);
+    m_positionOfCaret = m_selectionEnd;
+    return wxT("%");
+  }
+  wxString wordChars = wxT("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890_");
+
+  long left = m_positionOfCaret, right = m_positionOfCaret;
+  while (left > 0)
+  {
+    if(wordChars.Find(m_text.GetChar(left)) == -1) {
+      left++;
+      break;
+    }
+    left--;
+  }
+
+  while (right < m_text.length() )
+  {
+    if(wordChars.Find(m_text.GetChar(right)) == -1)
+      break;
+    right++;
+  }
+
+  if (left != right)
+  {
+    m_selectionStart = left;
+    m_selectionEnd = right;
+    m_positionOfCaret = m_selectionEnd;
+    return m_text.SubString(m_selectionStart, m_selectionEnd - 1);
+  }
+
+  return wxString(wxT("%"));
+}
+
 bool EditorCell::CopyToClipboard()
 {
   if (m_selectionStart == -1)
@@ -773,6 +1140,9 @@ bool EditorCell::CutToClipboard()
   if (m_selectionStart == -1)
     return false;
 
+  SaveValue();
+  m_saveValue = true;
+  m_containsChanges = true;
   CopyToClipboard();
 
   long start = MIN(m_selectionStart, m_selectionEnd);
@@ -782,7 +1152,7 @@ bool EditorCell::CutToClipboard()
            m_text.SubString(end, m_text.Length());
 
   m_selectionEnd = m_selectionStart = -1;
-
+  m_paren1 = m_paren2 = -1;
   m_width = m_height = m_maxDrop = m_center = -1;
 
   return true;
@@ -796,6 +1166,11 @@ void EditorCell::PasteFromClipboard()
     {
       wxTextDataObject obj;
       wxTheClipboard->GetData(obj);
+
+      SaveValue();
+      m_saveValue = true;
+      m_containsChanges = true;
+
       if (m_selectionStart > -1)
       {
         long start = MIN(m_selectionStart, m_selectionEnd);
@@ -812,6 +1187,7 @@ void EditorCell::PasteFromClipboard()
       m_positionOfCaret += data.Length();
       m_selectionEnd = m_positionOfCaret;
     }
+    wxTheClipboard->Close();
   }
 
   m_width = m_height = m_maxDrop = m_center = -1;
@@ -835,3 +1211,32 @@ wxString EditorCell::GetLineString(int line, int start, int end)
 
   return m_text.SubString(posStart, posEnd - 1);
 }
+
+void EditorCell::Undo()
+{
+  m_text = m_oldText;
+  m_positionOfCaret = m_oldPosition;
+  m_selectionStart = m_oldStart;
+  m_selectionEnd = m_oldEnd;
+  m_paren1 = m_paren2 = -1;
+  m_isDirty = true;
+  m_width = m_height = m_maxDrop = m_center = -1;
+}
+
+void EditorCell::SaveValue()
+{
+  m_oldText = m_text;
+  m_oldPosition = m_positionOfCaret;
+  m_oldStart = m_selectionStart;
+  m_oldEnd = m_selectionEnd;
+}
+
+bool EditorCell::CheckChanges()
+{
+  if (m_containsChanges != m_containsChangesCheck) {
+    m_containsChangesCheck = m_containsChanges;
+    return true;
+  }
+
+  return false;
+}
