@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2006 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -35,6 +35,8 @@ public:
   void RecalculateWidths(CellParser& parser, int fontsize, bool all);
   void Draw(CellParser& parser, wxPoint point, int fontsize, bool all);
   wxString ToString(bool all);
+  wxString ToTeX(bool all);
+	wxString ToXML(bool all);	//new!
   wxString GetDiffPart();
   void SelectInner(wxRect& rect, MathCell **first, MathCell **last);
   void IsMatrix(bool isMatrix)
@@ -43,6 +45,7 @@ public:
   }
   bool BreakUp();
   void Unbreak(bool all);
+  void SetParent(MathCell *parent, bool all);
 protected:
   MathCell *m_baseCell, *m_powCell;
   MathCell *m_open, *m_close, *m_exp, *m_last1, *m_last2;
