@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2006 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -34,6 +34,24 @@ FracCell::FracCell() : MathCell()
   m_divide = NULL;
 }
 
+void FracCell::SetParent(MathCell *parent, bool all)
+{
+  if (m_num != NULL)
+    m_num->SetParent(parent, true);
+  if (m_denom != NULL)
+    m_denom->SetParent(parent, true);
+  if (m_open1 != NULL)
+    m_open1->SetParent(parent, true);
+  if (m_close1 != NULL)
+    m_close2->SetParent(parent, true);
+  if (m_open2 != NULL)
+    m_open2->SetParent(parent, true);
+  if (m_close2 != NULL)
+    m_close2->SetParent(parent, true);
+
+  MathCell::SetParent(parent, all);
+}
+
 MathCell* FracCell::Copy(bool all)
 {
   FracCell* tmp = new FracCell;
@@ -103,8 +121,8 @@ void FracCell::RecalculateWidths(CellParser& parser, int fontsize, bool all)
   }
   else
   {
-    m_num->RecalculateWidths(parser, MAX(8, fontsize - 2), true);
-    m_denom->RecalculateWidths(parser, MAX(8, fontsize - 2), true);
+    m_num->RecalculateWidths(parser, MAX(MC_MIN_SIZE, fontsize - 2), true);
+    m_denom->RecalculateWidths(parser, MAX(MC_MIN_SIZE, fontsize - 2), true);
   }
   if (m_exponent && !m_isBroken)
   {
@@ -135,8 +153,8 @@ void FracCell::RecalculateSize(CellParser& parser, int fontsize, bool all)
   }
   else
   {
-    m_num->RecalculateSize(parser, MAX(8, fontsize - 2), true);
-    m_denom->RecalculateSize(parser, MAX(8, fontsize - 2), true);
+    m_num->RecalculateSize(parser, MAX(MC_MIN_SIZE, fontsize - 2), true);
+    m_denom->RecalculateSize(parser, MAX(MC_MIN_SIZE, fontsize - 2), true);
   }
   if (!m_exponent)
   {
@@ -177,8 +195,8 @@ void FracCell::Draw(CellParser& parser, wxPoint point, int fontsize, bool all)
       denom.y = num.y;
 
       dc.GetTextExtent(wxT("/"), &width, &height);
-      m_num->Draw(parser, num, MAX(8, fontsize - 2), true);
-      m_denom->Draw(parser, denom, MAX(8, fontsize - 2), true);
+      m_num->Draw(parser, num, MAX(MC_MIN_SIZE, fontsize - 2), true);
+      m_denom->Draw(parser, denom, MAX(MC_MIN_SIZE, fontsize - 2), true);
       dc.DrawText(wxT("/"),
                   point.x + m_num->GetFullWidth(scale),
                   point.y - m_num->GetMaxCenter() + SCALE_PX(2, scale));
@@ -188,11 +206,11 @@ void FracCell::Draw(CellParser& parser, wxPoint point, int fontsize, bool all)
       num.x = point.x + (m_width - m_num->GetFullWidth(scale)) / 2;
       num.y = point.y - m_num->GetMaxHeight() + m_num->GetMaxCenter() -
               SCALE_PX(2, scale);
-      m_num->Draw(parser, num, MAX(8, fontsize - 2), true);
+      m_num->Draw(parser, num, MAX(MC_MIN_SIZE, fontsize - 2), true);
 
       denom.x = point.x + (m_width - m_denom->GetFullWidth(scale)) / 2;
       denom.y = point.y + m_denom->GetMaxCenter() + SCALE_PX(2, scale);
-      m_denom->Draw(parser, denom, MAX(8, fontsize - 2), true);
+      m_denom->Draw(parser, denom, MAX(MC_MIN_SIZE, fontsize - 2), true);
       SetPen(parser);
       if (m_fracStyle != FC_CHOOSE)
         dc.DrawLine(point.x, point.y, point.x + m_width, point.y);
@@ -221,7 +239,7 @@ wxString FracCell::ToString(bool all)
     }
     else if (m_fracStyle == FC_CHOOSE)
     {
-      s = wxT("binom(") + m_num->ToString(true) + wxT(",") +
+      s = wxT("binomial(") + m_num->ToString(true) + wxT(",") +
           m_denom->ToString(true) + wxT(")");
     }
     else
@@ -248,6 +266,39 @@ wxString FracCell::ToString(bool all)
   return s;
 }
 
+wxString FracCell::ToTeX(bool all)
+{
+  wxString s;
+  if (!m_isBroken)
+  {
+    if (m_fracStyle == FC_CHOOSE)
+    {
+      s = wxT("\\pmatrix{") + m_num->ToTeX(true) + wxT("\\cr ") +
+          m_denom->ToTeX(true) + wxT("}");
+    }
+    else
+    {
+      s = wxT("\\frac{") + m_num->ToTeX(true) + wxT("}{") +
+          m_denom->ToTeX(true) + wxT("}");
+    }
+  }
+  if (m_fracStyle == FC_NORMAL)
+    s += MathCell::ToTeX(all);
+  return s;
+}
+
+wxString FracCell::ToXML(bool all)
+{
+	if( m_isBroken )
+		return wxEmptyString;
+	wxString s = ( m_fracStyle == FC_NORMAL || m_fracStyle == FC_DIFF )? 
+					_T("f"): _T("f line = \"no\"");
+	return _T("<") + s + _T("><r>") + 
+			m_num->ToXML(true) + _T("</r><r>") +
+			m_denom->ToXML(true) + _T("</r></f>") + 
+			MathCell::ToXML(all);
+}
+
 void FracCell::SelectInner(wxRect& rect, MathCell **first, MathCell **last)
 {
   *first = NULL;
@@ -291,7 +342,7 @@ void FracCell::SetupBreakUps()
   }
   else
   {
-    m_open1 = new TextCell(wxT("binom("));
+    m_open1 = new TextCell(wxT("binomial("));
     m_close1 = new TextCell(wxT("x"));
     m_open2 = new TextCell(wxT("x"));
     m_close2 = new TextCell(wxT(")"));
