@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2006 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@ IntCell::IntCell() : MathCell()
   m_under = NULL;
   m_over = NULL;
   m_var = NULL;
-  m_signSize = 20;
+  m_signSize = 50;
   m_signWidth = 18;
   m_signMiddle = 9;
   m_intStyle = INT_IDEF;
@@ -60,6 +60,20 @@ IntCell::~IntCell()
     delete m_next;
 }
 
+void IntCell::SetParent(MathCell *parent, bool all)
+{
+  if (m_base != NULL)
+    m_base->SetParent(parent, true);
+  if (m_under != NULL)
+    m_under->SetParent(parent, true);
+  if (m_over != NULL)
+    m_over->SetParent(parent, true);
+  if (m_var != NULL)
+    m_var->SetParent(parent, true);
+
+  MathCell::SetParent(parent, all);
+}
+
 MathCell* IntCell::Copy(bool all)
 {
   IntCell *tmp = new IntCell;
@@ -131,7 +145,7 @@ void IntCell::RecalculateWidths(CellParser& parser, int fontsize, bool all)
 {
   double scale = parser.GetScale();
 
-  m_signSize = SCALE_PX(20, scale);
+  m_signSize = SCALE_PX(50, scale);
   m_signWidth = SCALE_PX(18, scale);
   m_signMiddle = SCALE_PX(9, scale);
 
@@ -145,23 +159,27 @@ void IntCell::RecalculateWidths(CellParser& parser, int fontsize, bool all)
   m_over->RecalculateWidths(parser, MAX(8, fontsize - 5), true);
 
   m_signMiddle = MAX(m_signMiddle,
-                     m_under->GetFullWidth(scale) / 2 + SCALE_PX(5, scale));
+                     MAX(m_under->GetFullWidth(scale) / 2 + SCALE_PX(4, scale),
+			 m_over->GetFullWidth(scale) / 2 - SCALE_PX(4, scale)));
 
 #if defined __WXMSW__ || (wxUSE_UNICODE && WXM_UNICODE_GLYPHS)
   wxDC& dc = parser.GetDC();
   int fontsize1 = (int) ((INTEGRAL_FONT_SIZE * scale + 0.5));
   dc.SetFont(wxFont(fontsize1, wxMODERN,
-                    parser.IsItalic(TS_NORMAL_TEXT),
-                    parser.IsBold(TS_NORMAL_TEXT),
-                    parser.IsUnderlined(TS_NORMAL_TEXT),
+                    false, false, false,
                     parser.GetSymbolFontName()));
   dc.GetTextExtent(wxT(INTEGRAL_TOP), &m_charWidth, &m_charHeight);
 #endif
   int signOver = MAX(SCALE_PX(10, scale),
-                     m_over->GetFullWidth(scale) / 2 + SCALE_PX(5, scale));
-  m_signWidth = m_signMiddle + signOver;
+                     m_over->GetFullWidth(scale) / 2 + SCALE_PX(4, scale));
+
+  m_signWidth = m_signMiddle +
+    MAX(m_signMiddle,
+	MAX(m_over->GetFullWidth(scale) / 2 + SCALE_PX(4, scale),
+	    m_under->GetFullWidth(scale) / 2 - SCALE_PX(4, scale)));
+
   m_width = m_signWidth + m_base->GetFullWidth(scale) +
-            m_var->GetFullWidth(scale) - SCALE_PX(2, scale);
+            m_var->GetFullWidth(scale) + SCALE_PX(2, scale);
 
   MathCell::RecalculateWidths(parser, fontsize, all);
 }
@@ -170,26 +188,24 @@ void IntCell::RecalculateSize(CellParser& parser, int fontsize, bool all)
 {
   double scale = parser.GetScale();
 
-  m_under->RecalculateSize(parser, MAX(8, fontsize - 5), true);
-  m_over->RecalculateSize(parser, MAX(8, fontsize - 5), true);
+  m_under->RecalculateSize(parser, MAX(MC_MIN_SIZE, fontsize - 5), true);
+  m_over->RecalculateSize(parser, MAX(MC_MIN_SIZE, fontsize - 5), true);
   m_base->RecalculateSize(parser, fontsize, true);
   m_var->RecalculateSize(parser, fontsize, true);
 
-  m_signSize = MAX(m_signSize, m_base->GetMaxHeight() + SCALE_PX(30, scale));
-  m_signSize = MAX(m_signSize, m_var->GetMaxHeight() + SCALE_PX(30, scale));
   if (m_intStyle == INT_DEF)
   {
-    m_height = m_signSize + m_under->GetMaxHeight() + m_over->GetMaxHeight();
-    m_center = MAX((m_signSize + 1) / 2,
-                   m_base->GetMaxCenter() + SCALE_PX(15, scale)) +
-               m_over->GetMaxHeight();
+    m_center = MAX(m_over->GetMaxHeight() + SCALE_PX(4, scale) + m_signSize / 2,
+		   m_base->GetMaxCenter());
+    m_height = m_center +
+      MAX(m_under->GetMaxHeight() + SCALE_PX(4, scale) + m_signSize / 2,
+	  m_base->GetMaxDrop());
   }
   else
   {
-    m_height = m_signSize + SCALE_PX(6, scale);
-    m_center = MAX((m_signSize + 1) / 2,
-                   m_base->GetMaxCenter() + SCALE_PX(15, scale)) +
-               SCALE_PX(3, scale);
+    m_center = MAX(m_signSize / 2, m_base->GetMaxCenter());
+    m_height = m_center +
+      MAX(m_signSize / 2, m_base->GetMaxDrop());
   }
 
   MathCell::RecalculateSize(parser, fontsize, all);
@@ -207,29 +223,21 @@ void IntCell::Draw(CellParser& parser, wxPoint point, int fontsize, bool all)
     if (m_intStyle == INT_DEF)
     {
       under.x += m_signMiddle - m_under->GetFullWidth(scale) / 2 -
-                 SCALE_PX(5, scale);
-      under.y = point.y + (m_height - m_center) - m_under->GetMaxDrop();
-      m_under->Draw(parser, under, MAX(8, fontsize - 5), true);
+                 SCALE_PX(4, scale);
+      under.y = point.y + m_signSize / 2 + m_under->GetMaxCenter() + SCALE_PX(2, scale);
+      m_under->Draw(parser, under, MAX(MC_MIN_SIZE, fontsize - 5), true);
 
       over.x += m_signMiddle - m_over->GetFullWidth(scale) / 2 +
-                SCALE_PX(5, scale);
-      over.y = point.y - m_center + m_over->GetMaxCenter();
-      m_over->Draw(parser, over, MAX(8, fontsize - 5), true);
+                SCALE_PX(4, scale);
+      over.y = point.y - m_signSize/2 - m_over->GetMaxDrop() - SCALE_PX(2, scale);
+      m_over->Draw(parser, over, MAX(MC_MIN_SIZE, fontsize - 5), true);
     }
 
-    sign.y = sign.y - m_center + (m_signSize + 1) / 2;
-    if (m_intStyle == INT_DEF)
-      sign.y += m_over->GetMaxHeight();
-    else
-      sign.y += SCALE_PX(3, scale);
-
 #if defined __WXMSW__ || (wxUSE_UNICODE && WXM_UNICODE_GLYPHS)
     SetForeground(parser);
     int fontsize1 = (int) ((INTEGRAL_FONT_SIZE * scale + 0.5));
     dc.SetFont(wxFont(fontsize1, wxMODERN,
-               parser.IsItalic(TS_NORMAL_TEXT),
-               parser.IsBold(TS_NORMAL_TEXT),
-               parser.IsUnderlined(TS_NORMAL_TEXT),
+               false, false, false,
                parser.GetSymbolFontName()));
     dc.DrawText(wxT(INTEGRAL_TOP),
                 sign.x + m_signMiddle - m_charWidth / 2,
@@ -288,10 +296,10 @@ void IntCell::Draw(CellParser& parser, wxPoint point, int fontsize, bool all)
                 sign.y + (m_signSize + 1) / 2 - SCALE_PX(12, scale) + 1);
     UnsetPen(parser);
 #endif
-    base.x += m_signWidth - SCALE_PX(6, scale);
+    base.x += m_signWidth + SCALE_PX(2, scale);
     m_base->Draw(parser, base, fontsize, true);
 
-    var.x += m_signWidth + m_base->GetFullWidth(scale) - SCALE_PX(2, scale);
+    var.x += m_signWidth + m_base->GetFullWidth(scale) + SCALE_PX(2, scale);
     var.y += m_var->GetMaxCenter() - m_var->GetMaxHeight() / 2;
     m_var->Draw(parser, var, fontsize, true);
   }
@@ -336,7 +344,7 @@ wxString IntCell::ToTeX(bool all)
     s += wxT("_{") + from + wxT("}^{") + to + wxT("}");
   else
     s += wxT(" ");
-  
+
   s += m_base->ToTeX(true);
   s += m_var->ToTeX(true);
 
@@ -344,6 +352,28 @@ wxString IntCell::ToTeX(bool all)
   return s;
 }
 
+wxString IntCell::ToXML(bool all)
+{
+	MathCell* tmp = m_base;
+	wxString base = _T("<r>") + tmp->ToXML(true) + _T("</r>");
+
+	tmp = m_var;
+	wxString var = ( tmp == NULL )? wxEmptyString : _T("<r>");
+   	var += tmp->ToXML(true);
+	var += ( var == wxEmptyString )? wxEmptyString : _T("</r>");
+
+	tmp = m_under;
+	wxString from = _T("<r>") + tmp->ToXML(true) + _T("</r>");
+
+	tmp = m_over;
+	wxString to = _T("<r>") + tmp->ToXML(true) + _T("</r>");
+
+	return _T("<in>") + ( ( m_intStyle == INT_DEF )?
+					from + to + base + var :
+					base + var ) + _T("</in>") + 
+					MathCell::ToXML(all);
+}
+
 void IntCell::SelectInner(wxRect& rect, MathCell** first, MathCell** last)
 {
   *first = NULL;
