@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2006 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2008 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -17,6 +17,11 @@
 ///  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ///
 
+#include <wx/wx.h>
+#include <wx/config.h>
+#include <wx/tokenzr.h>
+#include <wx/sstream.h>
+
 #include "MathParser.h"
 
 #include "FracCell.h"
@@ -35,32 +40,133 @@
 #include "FunCell.h"
 #include "EditorCell.h"
 #include "ImgCell.h"
-
-#include <wx/wx.h>
-#include <wx/config.h>
+#include "SubSupCell.h"
+#include "SlideShowCell.h"
+#include "GroupCell.h"
 
 #define MAXLENGTH 50000
 
-MathParser::MathParser()
-{}
+MathParser::MathParser(wxString zipfile)
+{
+  m_ParserStyle = MC_TYPE_DEFAULT;
+  m_FracStyle = FC_NORMAL;
+  m_highlight = false;
+  if (zipfile.Length() > 0) {
+    m_fileSystem = new wxFileSystem();
+    m_fileSystem->ChangePathTo(wxT("file:") + zipfile + wxT("#zip:/"), true);
+  }
+  else
+    m_fileSystem = NULL;
+}
 
 MathParser::~MathParser()
-{}
+{
+  if (m_fileSystem)
+    delete m_fileSystem;
+}
 
-MathCell* MathParser::ParseFracTag(xmlNodePtr node)
+// ParseCellTag
+// This function is responsible for creating
+// a tree of groupcells when loading XML document.
+// Any changes in GroupCell structure or methods
+// has to be reflected here in order to ensure proper
+// loading of WXMX files.
+MathCell* MathParser::ParseCellTag(wxXmlNode* node)
+{
+  GroupCell *group = NULL;
+
+  // read hide status
+  bool hide = (node->GetPropVal(wxT("hide"), wxT("false")) == wxT("true")) ? true : false;
+
+  // read (group)cell type
+  wxString type = node->GetPropVal(wxT("type"), wxT("text"));
+
+  if (type == wxT("code")) {
+    group = new GroupCell(GC_TYPE_CODE);
+    wxXmlNode *children = node->GetChildren();
+    while (children) {
+      if (children->GetName() == wxT("input")) {
+        MathCell *editor = ParseTag(node->GetChildren());
+        group->SetEditableContent(editor->GetValue());
+        delete editor;
+      }
+      if (children->GetName() == wxT("output"))
+        group->AppendOutput(ParseTag(children->GetChildren()));
+      children = children->GetNext();
+    }
+  }
+  else if (type == wxT("image")) {
+    group = new GroupCell(GC_TYPE_IMAGE);
+    group->AppendOutput(ParseTag(node->GetChildren()));
+  }
+  else {
+    // text types
+    if (type == wxT("title"))
+      group = new GroupCell(GC_TYPE_TITLE);
+    else if (type == wxT("section"))
+      group = new GroupCell(GC_TYPE_SECTION);
+    else if (type == wxT("text"))
+      group = new GroupCell(GC_TYPE_TEXT);
+    else
+      return NULL;
+
+    MathCell *editor = ParseTag(node->GetChildren());
+    group->SetEditableContent(editor->GetValue());
+    delete editor;
+  }
+
+  group->SetParent(group, false);
+  group->Hide(hide);
+  return group;
+}
+
+MathCell* MathParser::ParseEditorTag(wxXmlNode* node)
+{
+  EditorCell *editor = new EditorCell();
+  wxString type = node->GetPropVal(wxT("type"), wxT("input"));
+  if (type == wxT("input"))
+    editor->SetType(MC_TYPE_INPUT);
+  else if (type == wxT("text"))
+    editor->SetType(MC_TYPE_TEXT);
+  else if (type == wxT("title"))
+    editor->SetType(MC_TYPE_TITLE);
+  else if (type == wxT("section"))
+    editor->SetType(MC_TYPE_SECTION);
+
+  wxString text = wxEmptyString;
+  wxXmlNode *line = node->GetChildren();
+  while (line) {
+    if (line->GetName() == wxT("line")) {
+      if (!text.IsEmpty())
+        text += wxT("\n");
+#if wxUSE_UNICODE
+      text += line->GetNodeContent();
+#else
+      wxString str = line->GetNodeContent();
+      wxString str1(str.wc_str(wxConvUTF8), *wxConvCurrent);
+      text += str1;
+#endif
+    }
+    line = line->GetNext();
+  } // end while
+  editor->SetValue(text);
+  return editor;
+}
+
+MathCell* MathParser::ParseFracTag(wxXmlNode* node)
 {
   FracCell *frac = new FracCell;
   frac->SetFracStyle(m_FracStyle);
   frac->SetHighlight(m_highlight);
-  xmlNodePtr child = node->children;
+  wxXmlNode* child = node->GetChildren();
   if (child)
   {
     frac->SetNum(ParseTag(child, false));
-    child = child->next;
+    child = child->GetNext();
     if (child)
     {
       frac->SetDenom(ParseTag(child, false));
-      if (node->properties != NULL)
+      if (node->GetProperties() != NULL)
       {
         frac->SetFracStyle(FC_CHOOSE);
       }
@@ -73,17 +179,17 @@ MathCell* MathParser::ParseFracTag(xmlNodePtr node)
   return NULL;
 }
 
-MathCell* MathParser::ParseDiffTag(xmlNodePtr node)
+MathCell* MathParser::ParseDiffTag(wxXmlNode* node)
 {
   DiffCell *diff = new DiffCell;
-  xmlNodePtr child = node->children;
+  wxXmlNode* child = node->GetChildren();
   if (child)
   {
     int fc = m_FracStyle;
     m_FracStyle = FC_DIFF;
     diff->SetDiff(ParseTag(child, false));
     m_FracStyle = fc;
-    child = child->next;
+    child = child->GetNext();
     if (child)
     {
       diff->SetBase(ParseTag(child, true));
@@ -95,16 +201,16 @@ MathCell* MathParser::ParseDiffTag(xmlNodePtr node)
   return NULL;
 }
 
-MathCell* MathParser::ParseSupTag(xmlNodePtr node)
+MathCell* MathParser::ParseSupTag(wxXmlNode* node)
 {
   ExptCell *expt = new ExptCell;
-  if (node->properties != NULL)
+  if (node->GetProperties() != NULL)
     expt->IsMatrix(true);
-  xmlNodePtr child = node->children;
+  wxXmlNode* child = node->GetChildren();
   if (child)
   {
     expt->SetBase(ParseTag(child, false));
-    child = child->next;
+    child = child->GetNext();
     if (child)
     {
       MathCell* power = ParseTag(child, false);
@@ -118,17 +224,47 @@ MathCell* MathParser::ParseSupTag(xmlNodePtr node)
   return NULL;
 }
 
-MathCell* MathParser::ParseSubTag(xmlNodePtr node)
+MathCell* MathParser::ParseSubSupTag(wxXmlNode* node)
+{
+  SubSupCell *subsup = new SubSupCell;
+  wxXmlNode* child = node->GetChildren();
+  if (child)
+  {
+    subsup->SetBase(ParseTag(child, false));
+    child = child->GetNext();
+    if (child)
+    {
+      MathCell* index = ParseTag(child, false);
+      index->SetExponentFlag();
+      subsup->SetIndex(index);
+      child = child->GetNext();
+      if (child)
+      {
+        MathCell* power = ParseTag(child, false);
+        power->SetExponentFlag();
+        subsup->SetExponent(power);
+        subsup->SetType(m_ParserStyle);
+        return subsup;
+      }
+    }
+  }
+  delete subsup;
+  return NULL;
+}
+
+MathCell* MathParser::ParseSubTag(wxXmlNode* node)
 {
   SubCell *sub = new SubCell;
-  xmlNodePtr child = node->children;
+  wxXmlNode* child = node->GetChildren();
   if (child)
   {
     sub->SetBase(ParseTag(child, false));
-    child = child->next;
+    child = child->GetNext();
     if (child)
     {
-      sub->SetIndex(ParseTag(child, false));
+      MathCell* index = ParseTag(child, false);
+      index->SetExponentFlag();
+      sub->SetIndex(index);
       sub->SetType(m_ParserStyle);
       return sub;
     }
@@ -137,15 +273,15 @@ MathCell* MathParser::ParseSubTag(xmlNodePtr node)
   return NULL;
 }
 
-MathCell* MathParser::ParseAtTag(xmlNodePtr node)
+MathCell* MathParser::ParseAtTag(wxXmlNode* node)
 {
   AtCell *at = new AtCell;
-  xmlNodePtr child = node->children;
+  wxXmlNode* child = node->GetChildren();
   if (child)
   {
     at->SetBase(ParseTag(child, false));
     at->SetHighlight(m_highlight);
-    child = child->next;
+    child = child->GetNext();
     if (child)
     {
       at->SetIndex(ParseTag(child, false));
@@ -157,14 +293,14 @@ MathCell* MathParser::ParseAtTag(xmlNodePtr node)
   return NULL;
 }
 
-MathCell* MathParser::ParseFunTag(xmlNodePtr node)
+MathCell* MathParser::ParseFunTag(wxXmlNode* node)
 {
   FunCell *fun = new FunCell;
-  xmlNodePtr child = node->children;
+  wxXmlNode* child = node->GetChildren();
   if (child)
   {
     fun->SetName(ParseTag(child, false));
-    child = child->next;
+    child = child->GetNext();
     if (child)
     {
       fun->SetType(m_ParserStyle);
@@ -176,45 +312,17 @@ MathCell* MathParser::ParseFunTag(xmlNodePtr node)
   return NULL;
 }
 
-/************
- * Check if n is an integer or a bflat (format 3.14b0)
- */
-
-bool IsNumber(wxString n)
-{
-  unsigned int n_dots = 0, n_b = 0, i = 0;
-  char c;
-  for (i = 0; i < n.Length(); i++)
-  {
-    c = n.GetChar(i);
-    if (c == '.')
-    {
-      n_dots++;
-      if (n_dots > 1)
-        break;
-    }
-    else if (c == 'b')
-    {
-      n_b++;
-      if (n_b > 1)
-        break;
-    }
-    else if (c < '0' || c > '9')
-      break;
-  }
-  if (i < n.Length())
-    return false;
-  return true;
-}
-
-MathCell* MathParser::ParseText(xmlNodePtr node, int style)
+MathCell* MathParser::ParseText(wxXmlNode* node, int style)
 {
   TextCell* cell = new TextCell;
-  if (node != NULL && node->content != NULL)
+  wxString str;
+  if (node != NULL && (str = node->GetContent()) != wxEmptyString)
   {
-    wxString str((const char*)(node->content), wxConvUTF8);
-    str = ToLocal(str);
-    if (IsNumber(str))
+#if !wxUSE_UNICODE
+    wxString str1(str.wc_str(wxConvUTF8), *wxConvCurrent);
+    str = str1;
+#endif
+    if (style == TS_NUMBER)
     {
       if (str.Length() > 100) // This could be made configurable.
         str = str.Left(30) + wxString::Format(wxT("[%d digits]"), str.Length() - 60) + str.Right(30);
@@ -227,9 +335,9 @@ MathCell* MathParser::ParseText(xmlNodePtr node, int style)
   return cell;
 }
 
-MathCell* MathParser::ParseSqrtTag(xmlNodePtr node)
+MathCell* MathParser::ParseSqrtTag(wxXmlNode* node)
 {
-  xmlNodePtr child = node->children;
+  wxXmlNode* child = node->GetChildren();
   SqrtCell* cell = new SqrtCell;
   cell->SetInner(ParseTag(child, true));
   cell->SetType(m_ParserStyle);
@@ -237,9 +345,9 @@ MathCell* MathParser::ParseSqrtTag(xmlNodePtr node)
   return cell;
 }
 
-MathCell* MathParser::ParseAbsTag(xmlNodePtr node)
+MathCell* MathParser::ParseAbsTag(wxXmlNode* node)
 {
-  xmlNodePtr child = node->children;
+  wxXmlNode* child = node->GetChildren();
   AbsCell* cell = new AbsCell;
   cell->SetInner(ParseTag(child, true));
   cell->SetType(m_ParserStyle);
@@ -247,29 +355,29 @@ MathCell* MathParser::ParseAbsTag(xmlNodePtr node)
   return cell;
 }
 
-MathCell* MathParser::ParseParenTag(xmlNodePtr node)
+MathCell* MathParser::ParseParenTag(wxXmlNode* node)
 {
-  xmlNodePtr child = node->children;
+  wxXmlNode* child = node->GetChildren();
   ParenCell* cell = new ParenCell;
   cell->SetInner(ParseTag(child, true), m_ParserStyle);
   cell->SetHighlight(m_highlight);
-  if (node->properties != NULL)
+  if (node->GetProperties() != NULL)
     cell->SetPrint(false);
   return cell;
 }
 
-MathCell* MathParser::ParseLimitTag(xmlNodePtr node)
+MathCell* MathParser::ParseLimitTag(wxXmlNode* node)
 {
   LimitCell *limit = new LimitCell;
-  xmlNodePtr child = node->children;
+  wxXmlNode* child = node->GetChildren();
   if (child)
   {
     limit->SetName(ParseTag(child, false));
-    child = child->next;
+    child = child->GetNext();
     if (child)
     {
       limit->SetUnder(ParseTag(child, false));
-      child = child->next;
+      child = child->GetNext();
       if (child)
       {
         limit->SetBase(ParseTag(child, false));
@@ -282,21 +390,21 @@ MathCell* MathParser::ParseLimitTag(xmlNodePtr node)
   return NULL;
 }
 
-MathCell* MathParser::ParseSumTag(xmlNodePtr node)
+MathCell* MathParser::ParseSumTag(wxXmlNode* node)
 {
   SumCell *sum = new SumCell;
-  xmlNodePtr child = node->children;
-  if (node->properties != NULL)
+  wxXmlNode* child = node->GetChildren();
+  if (node->GetProperties() != NULL)
     sum->SetSumStyle(SM_PROD);
   sum->SetHighlight(m_highlight);
   if (child)
   {
     sum->SetUnder(ParseTag(child, false));
-    child = child->next;
+    child = child->GetNext();
     if (child)
     {
       sum->SetOver(ParseTag(child, false));
-      child = child->next;
+      child = child->GetNext();
       if (child)
       {
         sum->SetBase(ParseTag(child, false));
@@ -309,26 +417,26 @@ MathCell* MathParser::ParseSumTag(xmlNodePtr node)
   return NULL;
 }
 
-MathCell* MathParser::ParseIntTag(xmlNodePtr node)
+MathCell* MathParser::ParseIntTag(wxXmlNode* node)
 {
   IntCell *in = new IntCell;
-  xmlNodePtr child = node->children;
+  wxXmlNode* child = node->GetChildren();
   in->SetHighlight(m_highlight);
-  if (node->properties == NULL)
+  if (node->GetProperties() == NULL)
   {
     in->SetIntStyle(INT_DEF);
     if (child)
     {
       in->SetUnder(ParseTag(child, false));
-      child = child->next;
+      child = child->GetNext();
       if (child)
       {
         in->SetOver(ParseTag(child, false));
-        child = child->next;
+        child = child->GetNext();
         if (child)
         {
           in->SetBase(ParseTag(child, false));
-          child = child->next;
+          child = child->GetNext();
           if (child)
           {
             in->SetVar(ParseTag(child, true));
@@ -344,7 +452,7 @@ MathCell* MathParser::ParseIntTag(xmlNodePtr node)
     if (child)
     {
       in->SetBase(ParseTag(child, false));
-      child = child->next;
+      child = child->GetNext();
       if (child)
       {
         in->SetVar(ParseTag(child, true));
@@ -357,68 +465,68 @@ MathCell* MathParser::ParseIntTag(xmlNodePtr node)
   return NULL;
 }
 
-MathCell* MathParser::ParseTableTag(xmlNodePtr node)
+MathCell* MathParser::ParseTableTag(wxXmlNode* node)
 {
   MatrCell *matrix = new MatrCell;
   matrix->SetHighlight(m_highlight);
-  
-  if (node->properties != NULL)
+
+  if (node->GetProperties() != NULL)
     matrix->SetSpecialFlag(true);
-  
-  xmlNodePtr rows = node->children;
+
+  wxXmlNode* rows = node->GetChildren();
   while (rows)
   {
     matrix->NewRow();
-    xmlNodePtr cells = rows->children;
+    wxXmlNode* cells = rows->GetChildren();
     while (cells)
     {
       matrix->NewColumn();
       matrix->AddNewCell(ParseTag(cells, false));
-      cells = cells->next;
+      cells = cells->GetNext();
     }
-    rows = rows->next;
+    rows = rows->GetNext();
   }
   matrix->SetType(m_ParserStyle);
   matrix->SetDimension();
   return matrix;
 }
 
-MathCell* MathParser::ParseTag(xmlNodePtr node, bool all)
+MathCell* MathParser::ParseTag(wxXmlNode* node, bool all)
 {
   //  wxYield();
+  MathCell* tmp = NULL;
   MathCell* cell = NULL;
   while (node)
   {
     // Parse tags
-    if (node->type == XML_ELEMENT_NODE)
+    if (node->GetType() == wxXML_ELEMENT_NODE)
     {
-      wxString tagName((const char*)(node->name), wxConvUTF8);
-      tagName = ToLocal(tagName);
+      wxString tagName(node->GetName());
 
       if (tagName == wxT("v"))
       {               // Variables (atoms)
         if (cell == NULL)
-          cell = ParseText(node->children, TS_VARIABLES);
+          cell = ParseText(node->GetChildren(), TS_VARIABLE);
         else
-          cell->AppendCell(ParseText(node->children, TS_VARIABLES));
+          cell->AppendCell(ParseText(node->GetChildren(), TS_VARIABLE));
       }
       else if (tagName == wxT("t"))
       {          // Other text
         if (cell == NULL)
-          cell = ParseText(node->children, TS_NORMAL_TEXT);
+          cell = ParseText(node->GetChildren(), TS_DEFAULT);
         else
-          cell->AppendCell(ParseText(node->children, TS_NORMAL_TEXT));
+          cell->AppendCell(ParseText(node->GetChildren(), TS_DEFAULT));
       }
       else if (tagName == wxT("n"))
       {          // Numbers
         if (cell == NULL)
-          cell = ParseText(node->children, TS_NUMBER);
+          cell = ParseText(node->GetChildren(), TS_NUMBER);
         else
-          cell->AppendCell(ParseText(node->children, TS_NUMBER));
+          cell->AppendCell(ParseText(node->GetChildren(), TS_NUMBER));
       }
       else if (tagName == wxT("h"))
       {          // Hidden cells (*)
-        MathCell* tmp = ParseText(node->children);
+        MathCell* tmp = ParseText(node->GetChildren());
         tmp->m_isHidden = true;
         if (cell == NULL)
           cell = tmp;
@@ -462,7 +570,7 @@ MathCell* MathParser::ParseTag(xmlNodePtr node, bool all)
       }
       else if (tagName == wxT("g"))
       {          // Greek constants
-        MathCell* tmp = ParseText(node->children, TS_GREEK_CONSTANT);
+        MathCell* tmp = ParseText(node->GetChildren(), TS_GREEK_CONSTANT);
         if (cell == NULL)
           cell = tmp;
         else
@@ -470,7 +578,15 @@ MathCell* MathParser::ParseTag(xmlNodePtr node, bool all)
       }
       else if (tagName == wxT("s"))
       {          // Special constants %e,...
-        MathCell* tmp = ParseText(node->children, TS_SPECIAL_CONSTANT);
+        MathCell* tmp = ParseText(node->GetChildren(), TS_SPECIAL_CONSTANT);
+        if (cell == NULL)
+          cell = tmp;
+        else
+          cell->AppendCell(tmp);
+      }
+      else if (tagName == wxT("fnm"))
+      {         // Function names
+        MathCell* tmp = ParseText(node->GetChildren(), TS_FUNCTION);
         if (cell == NULL)
           cell = tmp;
         else
@@ -525,6 +641,13 @@ MathCell* MathParser::ParseTag(xmlNodePtr node, bool all)
         else
           cell->AppendCell(ParseAbsTag(node));
       }
+      else if (tagName == wxT("ie"))
+      {
+        if (cell == NULL)
+          cell = ParseSubSupTag(node);
+        else
+          cell->AppendCell(ParseSubSupTag(node));
+      }
       else if (tagName == wxT("lm"))
       {
         if (cell == NULL)
@@ -535,9 +658,9 @@ MathCell* MathParser::ParseTag(xmlNodePtr node, bool all)
       else if (tagName == wxT("r"))
       {
         if (cell == NULL)
-          cell = ParseTag(node->children);
+          cell = ParseTag(node->GetChildren());
         else
-          cell->AppendCell(ParseTag(node->children));
+          cell->AppendCell(ParseTag(node->GetChildren()));
       }
       else if (tagName == wxT("tb"))
       {
@@ -548,7 +671,7 @@ MathCell* MathParser::ParseTag(xmlNodePtr node, bool all)
       }
       else if (tagName == wxT("mth"))
       {
-        MathCell *tmp = ParseTag(node->children);
+        MathCell *tmp = ParseTag(node->GetChildren());
         if (tmp != NULL)
           tmp->ForceBreakLine(true);
         else
@@ -561,9 +684,7 @@ MathCell* MathParser::ParseTag(xmlNodePtr node, bool all)
       else if (tagName == wxT("lbl"))
       {
         int oldPS = m_ParserStyle;
-        m_ParserStyle = MC_TYPE_LABEL;
-        MathCell* tmp = ParseTag(node->children);
-        m_ParserStyle = oldPS;
+        MathCell* tmp = ParseText(node->GetChildren(), TS_LABEL);
         tmp->ForceBreakLine(true);
         if (cell == NULL)
           cell = tmp;
@@ -572,7 +693,7 @@ MathCell* MathParser::ParseTag(xmlNodePtr node, bool all)
       }
       else if (tagName == wxT("st"))
       {
-        MathCell* tmp = ParseText(node->children, TS_STRING);
+        MathCell* tmp = ParseText(node->GetChildren(), TS_STRING);
         if (cell == NULL)
           cell = tmp;
         else
@@ -582,7 +703,7 @@ MathCell* MathParser::ParseTag(xmlNodePtr node, bool all)
       {
         bool highlight = m_highlight;
         m_highlight = true;
-        MathCell* tmp = ParseTag(node->children);
+        MathCell* tmp = ParseTag(node->GetChildren());
         m_highlight = highlight;
         if (cell == NULL)
           cell = tmp;
@@ -591,23 +712,59 @@ MathCell* MathParser::ParseTag(xmlNodePtr node, bool all)
       }
       else if (tagName == wxT("img"))
       {
-        wxString filename((const char*)(node->children->content), wxConvUTF8);
-        filename = ToLocal(filename);
-        
-        ImgCell *tmp = new ImgCell;
-        tmp->LoadImage(filename);
-        wxRemoveFile(filename);
+        wxString filename(node->GetChildren()->GetContent());
+
+        ImgCell *tmp;
+
+        if (m_fileSystem) // loading from zip
+          tmp = new ImgCell(filename, false, m_fileSystem);
+        else if (node->GetPropVal(wxT("del"), wxT("yes")) != wxT("no"))
+          tmp = new ImgCell(filename, true, NULL);
+        else
+          tmp = new ImgCell(filename, false, NULL);
+
         if (cell == NULL)
           cell = tmp;
         else
           cell->AppendCell(tmp);
       }
-      else if (node->children)
+      else if (tagName == wxT("slide"))
+      {
+        SlideShow *tmp = new SlideShow(m_fileSystem);
+        wxString str(node->GetChildren()->GetContent());
+        wxArrayString images;
+        wxStringTokenizer tokens(str, wxT(";"));
+        while (tokens.HasMoreTokens()) {
+          wxString token = tokens.GetNextToken();
+          if (token.Length())
+            images.Add(token);
+        }
+        tmp->LoadImages(images);
+        if (cell == NULL)
+          cell = tmp;
+        else
+          cell->AppendCell(tmp);
+      }
+      else if (tagName == wxT("editor"))
       {
         if (cell == NULL)
-          cell = ParseTag(node->children);
+          cell = ParseEditorTag(node);
         else
-          cell->AppendCell(ParseTag(node->children));
+          cell->AppendCell(ParseEditorTag(node));
+      }
+      else if (tagName == wxT("cell"))
+      {
+        if (cell == NULL)
+          cell = ParseCellTag(node);
+        else
+          cell->AppendCell(ParseCellTag(node));
+      }
+      else if (node->GetChildren())
+      {
+        if (cell == NULL)
+          cell = ParseTag(node->GetChildren());
+        else
+          cell->AppendCell(ParseTag(node->GetChildren()));
       }
     }
     // Parse text
@@ -620,8 +777,17 @@ MathCell* MathParser::ParseTag(xmlNodePtr node, bool all)
     }
     if (!all)
       break;
-    node = node->next;
+
+    if (tmp == NULL)
+      tmp = cell;
+    else
+      cell = cell->m_next;
+
+    node = node->GetNext();
   }
+
+  if (tmp != NULL)
+    return tmp;
   return cell;
 }
 
@@ -642,23 +808,22 @@ MathCell* MathParser::ParseLine(wxString s, int style)
 
   if (s.Length() < MAXLENGTH || showLong)
   {
-    s = ToUnicode(s);
+
+    wxXmlDocument xml;
 
 #if wxUSE_UNICODE
-    char *buf;
-    wxWX2MBbuf tmp = wxConvertWX2MB(s.wx_str());
-    buf = strdup(tmp);
-    xmlDocPtr doc = xmlParseMemory(buf, strlen(buf));
-    free(buf);
+    wxStringInputStream xmlStream(s);
 #else
-    xmlDocPtr doc = xmlParseMemory(s.c_str(), s.Length());
+    wxString su(s.wc_str(*wxConvCurrent), wxConvUTF8);
+    wxStringInputStream xmlStream(su);
 #endif
 
+    xml.Load(xmlStream);
+
+    wxXmlNode *doc = xml.GetRoot();
+
     if (doc != NULL)
-    {
-      cell = ParseTag(xmlDocGetRootElement(doc));
-      xmlFreeDoc(doc);
-    }
+      cell = ParseTag(doc->GetChildren());
   }
   else
   {
@@ -667,21 +832,3 @@ MathCell* MathParser::ParseLine(wxString s, int style)
   }
   return cell;
 }
-
-wxString MathParser::ToUnicode(wxString s)
-{
-#if wxUSE_UNICODE
-  return s;
-#else
-  return wxString(s.wc_str(wxConvLocal), wxConvUTF8);
-#endif
-}
-
-wxString MathParser::ToLocal(wxString s)
-{
-#if wxUSE_UNICODE
-  return s;
-#else
-  return wxString(s.wc_str(wxConvUTF8), wxConvLocal);
-#endif
-}
