@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2006 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -52,7 +52,9 @@ public:
   void SelectInner(wxRect& rect, MathCell** first, MathCell** last);
   wxString ToString(bool all);
   wxString ToTeX(bool all);
+	wxString ToXML(bool all);	// new!!
   void SetSpecialFlag(bool special) { m_specialMatrix = special; }
+  void SetParent(MathCell *parent, bool all);
 protected:
   int m_matWidth;
   int m_matHeight;
