@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2006 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -57,6 +57,18 @@ ParenCell::~ParenCell()
   delete m_close;
 }
 
+void ParenCell::SetParent(MathCell *parent, bool all)
+{
+  if (m_innerCell != NULL)
+    m_innerCell->SetParent(parent, true);
+  if (m_open != NULL)
+    m_open->SetParent(parent, true);
+  if (m_close != NULL)
+    m_close->SetParent(parent, true);
+
+  MathCell::SetParent(parent, all);
+}
+
 MathCell* ParenCell::Copy(bool all)
 {
   ParenCell *tmp = new ParenCell;
@@ -84,9 +96,6 @@ void ParenCell::SetInner(MathCell *inner, int type)
   m_innerCell = inner;
   m_type = type;
 
-
-  m_open->m_nextToDraw = m_innerCell;
-  m_open->m_previousToDraw = this;
   while (inner->m_next != NULL)
     inner = inner->m_next;
   m_last1 = inner;
@@ -104,9 +113,9 @@ void ParenCell::RecalculateWidths(CellParser& parser, int fontsize, bool all)
   wxDC& dc = parser.GetDC();
   int fontsize1 = (int) ((PAREN_FONT_SIZE * scale + 0.5));
   dc.SetFont(wxFont(fontsize1, wxMODERN,
-                    parser.IsItalic(TS_NORMAL_TEXT),
-                    parser.IsBold(TS_NORMAL_TEXT),
-                    parser.IsUnderlined(TS_NORMAL_TEXT),
+                    parser.IsItalic(TS_DEFAULT),
+                    parser.IsBold(TS_DEFAULT),
+                    parser.IsUnderlined(TS_DEFAULT),
                     parser.GetSymbolFontName()));
   dc.GetTextExtent(wxT(PAREN_LEFT_TOP), &m_charWidth, &m_charHeight);
   m_width = m_innerCell->GetFullWidth(scale) + 2*m_charWidth;
@@ -159,6 +168,7 @@ void ParenCell::Draw(CellParser& parser, wxPoint point, int fontsize, bool all)
 
 #if defined __WXMSW__
     int fontsize1 = (int) ((PAREN_FONT_SIZE * scale + 0.5));
+    SetForeground(parser);
     if (m_height < (3*m_charHeight)/2)
     {
       fontsize1 = (int) ((fontsize * scale + 0.5));
@@ -176,7 +186,6 @@ void ParenCell::Draw(CellParser& parser, wxPoint point, int fontsize, bool all)
     }
     else
     {
-      SetForeground(parser);
       dc.SetFont(wxFont(fontsize1, wxMODERN,
                         false,
                         false,
@@ -219,6 +228,7 @@ void ParenCell::Draw(CellParser& parser, wxPoint point, int fontsize, bool all)
     }
 #elif (wxUSE_UNICODE && WXM_UNICODE_GLYPHS)
     int fontsize1 = (int) ((PAREN_FONT_SIZE * scale + 0.5));
+    SetForeground(parser);
     if (m_height < (3*m_charHeight)/2)
     {
       fontsize1 = (int) ((fontsize * scale + 0.5));
@@ -236,7 +246,6 @@ void ParenCell::Draw(CellParser& parser, wxPoint point, int fontsize, bool all)
     }
     else if (m_height < (5*m_charHeight)/2)
     {
-      SetForeground(parser);
       dc.SetFont(wxFont(fontsize1, wxMODERN,
                         false,
                         false,
@@ -257,7 +266,6 @@ void ParenCell::Draw(CellParser& parser, wxPoint point, int fontsize, bool all)
     }
     else
     {
-      SetForeground(parser);
       dc.SetFont(wxFont(fontsize1, wxMODERN,
                         false,
                         false,
@@ -360,6 +368,15 @@ wxString ParenCell::ToTeX(bool all)
   return s;
 }
 
+wxString ParenCell::ToXML(bool all)
+{
+	if( m_isBroken )
+		return wxEmptyString;
+	wxString s = m_innerCell->ToXML(true);
+	return ( ( m_print )? _T("<p>") + s + _T("</p>") : s ) +
+			MathCell::ToXML(all);
+}
+
 void ParenCell::SelectInner(wxRect& rect, MathCell **first, MathCell **last)
 {
   *first = NULL;
