@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2006 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -41,10 +41,10 @@ private:
   void OnButton(wxCommandEvent& event);
   void OnCombobox(wxCommandEvent& event);
   void OnFileBrowse(wxCommandEvent& event);
+  void OnPopupMenu(wxCommandEvent& event);
   void Parse(wxString in);
 protected:
   int type;
-  wxStaticText* label_1;
   wxStaticText* label_2;
   BTextCtrl* text_ctrl_1;
   wxButton* button_3;
@@ -60,7 +60,6 @@ protected:
   BTextCtrl* text_ctrl_6;
   wxStaticText* label_8;
   BTextCtrl* text_ctrl_7;
-  wxCheckBox* checkbox_1;
   wxStaticText* label_9;
   wxComboBox* combo_box_1;
   wxStaticText* label_10;
@@ -76,23 +75,22 @@ protected:
   DECLARE_EVENT_TABLE()
 };
 
-
-class Plot2dPar: public wxDialog
+class Plot2DPar: public wxDialog
 {
 public:
-  Plot2dPar(wxWindow* parent, int id, const wxString& title,
+  Plot2DPar(wxWindow* parent, int id, const wxString& title,
             const wxPoint& pos = wxDefaultPosition,
             const wxSize& size = wxDefaultSize, long style = wxDEFAULT_DIALOG_STYLE);
   void SetValue(wxString s)
   {
     text_ctrl_1->SetValue(s);
+    text_ctrl_1->SetSelection(-1, -1);
   }
   wxString GetValue();
 private:
   void set_properties();
   void do_layout();
 protected:
-  wxStaticText* label_1;
   wxStaticText* label_2;
   wxStaticText* label_3;
   BTextCtrl* text_ctrl_1;
@@ -103,9 +101,37 @@ protected:
   wxStaticText* label_6;
   BTextCtrl* text_ctrl_4;
   BTextCtrl* text_ctrl_5;
+  wxStaticText* label_7;
+  wxSpinCtrl* spin_ctrl_1;
   wxStaticLine* static_line_1;
   wxButton* button_1;
   wxButton* button_2;
 };
 
+class Plot2DDiscrete: public wxDialog
+{
+public:
+  Plot2DDiscrete(wxWindow* parent, int id, const wxString& title,
+            const wxPoint& pos = wxDefaultPosition,
+            const wxSize& size = wxDefaultSize, long style = wxDEFAULT_DIALOG_STYLE);
+  void SetValue(wxString s)
+  {
+    text_ctrl_1->SetValue(s);
+    text_ctrl_1->SetSelection(-1, -1);
+  }
+  wxString GetValue();
+private:
+  void set_properties();
+  void do_layout();
+protected:
+  wxStaticText* label_2;
+  wxStaticText* label_3;
+  BTextCtrl* text_ctrl_1;
+  BTextCtrl* text_ctrl_2;
+  wxStaticLine* static_line_1;
+  wxButton* button_1;
+  wxButton* button_2;
+};
+
+
 #endif // PLOT2DWIZ_H
