@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2006 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -38,33 +38,34 @@ Plot3DWiz::Plot3DWiz(wxWindow* parent, int id,
                      const wxSize& size, long style):
     wxDialog(parent, id, title, pos, size, wxDEFAULT_DIALOG_STYLE)
 {
-  label_1 = new wxStaticText(this, -1, _("Plot 3D"));
   label_2 = new wxStaticText(this, -1, _("Expression"));
   text_ctrl_1 = new BTextCtrl(this, -1, wxEmptyString, wxDefaultPosition);
   label_3 = new wxStaticText(this, -1, _("Variable:"));
   text_ctrl_2 = new BTextCtrl(this, -1, wxT("x"), wxDefaultPosition,
                               wxSize(40, -1));
-  label_4 = new wxStaticText(this, -1, _("from:"));
+  label_4 = new wxStaticText(this, -1, _("From:"));
   text_ctrl_3 = new BTextCtrl(this, -1, wxEmptyString, wxDefaultPosition,
                               wxSize(70, -1));
-  label_5 = new wxStaticText(this, -1, _("to:"));
+  label_5 = new wxStaticText(this, -1, _("To:"));
   text_ctrl_4 = new BTextCtrl(this, -1, wxEmptyString, wxDefaultPosition,
                               wxSize(70, -1));
   label_6 = new wxStaticText(this, -1, _("Variable:"));
   text_ctrl_5 = new BTextCtrl(this, -1, wxT("y"), wxDefaultPosition,
                               wxSize(40, -1));
-  label_7 = new wxStaticText(this, -1, _("from:"));
+  label_7 = new wxStaticText(this, -1, _("From:"));
   text_ctrl_6 = new BTextCtrl(this, -1, wxEmptyString, wxDefaultPosition,
                               wxSize(70, -1));
-  label_8 = new wxStaticText(this, -1, _("to:"));
+  label_8 = new wxStaticText(this, -1, _("To:"));
   text_ctrl_7 = new BTextCtrl(this, -1, wxEmptyString, wxDefaultPosition,
                               wxSize(70, -1));
   label_9 = new wxStaticText(this, -1, _("Grid:"));
   text_ctrl_8 = new wxSpinCtrl(this, -1, wxEmptyString, wxDefaultPosition,
-                               wxSize(70, -1), wxSP_ARROW_KEYS, 0, 100);
+                               wxSize(70, -1), wxSP_ARROW_KEYS, 0, 100, 30);
+  text_ctrl_8->SetValue(30);
   label_10 = new wxStaticText(this, -1, wxT("x"));
   text_ctrl_9 = new wxSpinCtrl(this, -1, wxEmptyString, wxDefaultPosition,
-                               wxSize(70, -1), wxSP_ARROW_KEYS, 0, 100);
+                               wxSize(70, -1), wxSP_ARROW_KEYS, 0, 100, 30);
+  text_ctrl_9->SetValue(30);
   label_11 = new wxStaticText(this, -1, _("Format:"));
   const wxString combo_box_1_choices[] =
     {
@@ -115,16 +116,13 @@ Plot3DWiz::Plot3DWiz(wxWindow* parent, int id,
 void Plot3DWiz::set_properties()
 {
   SetTitle(_("Plot 3D"));
-  label_1->SetFont(wxFont(20, wxROMAN, wxITALIC, wxNORMAL, 0, wxEmptyString));
   text_ctrl_3->SetValue(wxT("-5"));
   text_ctrl_4->SetValue(wxT("5"));
   text_ctrl_6->SetValue(wxT("-5"));
   text_ctrl_7->SetValue(wxT("5"));
-  text_ctrl_8->SetValue(wxT("30"));
-  text_ctrl_9->SetValue(wxT("30"));
 
   button_3->SetToolTip(_("Browse"));
-  
+
   bool pm3dValue = false;
 #if defined __WXMSW__
   button_1->SetDefault();
@@ -132,65 +130,64 @@ void Plot3DWiz::set_properties()
 #else
   button_2->SetDefault();
 #endif
-  
+
   int selection = 1;
   wxConfig::Get()->Read(wxT("Wiz/Plot3D/format"), &selection);
   wxConfig::Get()->Read(wxT("Wiz/Plot3D/pm3d"), &pm3dValue);
-  
+
   combo_box_1->SetSelection(selection);
   check_box_1->SetValue(pm3dValue);
-  
+
   text_ctrl_1->SetFocus();
 }
 
 
 void Plot3DWiz::do_layout()
 {
-  wxFlexGridSizer* grid_sizer_1 = new wxFlexGridSizer(4, 1, 3, 3);
+  wxFlexGridSizer* grid_sizer_1 = new wxFlexGridSizer(4, 1, 0, 0);
   wxBoxSizer* sizer_1 = new wxBoxSizer(wxHORIZONTAL);
-  wxFlexGridSizer* grid_sizer_2 = new wxFlexGridSizer(7, 2, 3, 3);
+  wxFlexGridSizer* grid_sizer_2 = new wxFlexGridSizer(7, 2, 0, 0);
   wxBoxSizer* sizer_2 = new wxBoxSizer(wxHORIZONTAL);
   wxBoxSizer* sizer_3 = new wxBoxSizer(wxHORIZONTAL);
   wxBoxSizer* sizer_4 = new wxBoxSizer(wxHORIZONTAL);
   wxBoxSizer* sizer_5 = new wxBoxSizer(wxHORIZONTAL);
   wxBoxSizer* sizer_6 = new wxBoxSizer(wxHORIZONTAL);
-  grid_sizer_1->Add(label_1, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 2);
-  grid_sizer_2->Add(label_2, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  grid_sizer_2->Add(text_ctrl_1, 1, wxEXPAND | wxALL, 2);
-  grid_sizer_2->Add(label_3, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  sizer_2->Add(text_ctrl_2, 0, wxALL, 2);
-  sizer_2->Add(label_4, 0, wxALL | wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL, 2);
-  sizer_2->Add(text_ctrl_3, 0, wxALL, 2);
-  sizer_2->Add(label_5, 0, wxALL | wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL, 2);
-  sizer_2->Add(text_ctrl_4, 0, wxALL, 2);
+  grid_sizer_2->Add(label_2, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  grid_sizer_2->Add(text_ctrl_1, 1, wxEXPAND | wxALL, 5);
+  grid_sizer_2->Add(label_3, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  sizer_2->Add(text_ctrl_2, 0, wxALL, 5);
+  sizer_2->Add(label_4, 0, wxALL | wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL, 5);
+  sizer_2->Add(text_ctrl_3, 0, wxALL, 5);
+  sizer_2->Add(label_5, 0, wxALL | wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL, 5);
+  sizer_2->Add(text_ctrl_4, 0, wxALL, 5);
   grid_sizer_2->Add(sizer_2, 1, wxEXPAND, 0);
-  grid_sizer_2->Add(label_6, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  sizer_3->Add(text_ctrl_5, 0, wxALL, 2);
-  sizer_3->Add(label_7, 0, wxALL | wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL, 2);
-  sizer_3->Add(text_ctrl_6, 0, wxALL, 2);
-  sizer_3->Add(label_8, 0, wxALL | wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL, 2);
-  sizer_3->Add(text_ctrl_7, 0, wxALL, 2);
+  grid_sizer_2->Add(label_6, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  sizer_3->Add(text_ctrl_5, 0, wxALL, 5);
+  sizer_3->Add(label_7, 0, wxALL | wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL, 5);
+  sizer_3->Add(text_ctrl_6, 0, wxALL, 5);
+  sizer_3->Add(label_8, 0, wxALL | wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL, 5);
+  sizer_3->Add(text_ctrl_7, 0, wxALL, 5);
   grid_sizer_2->Add(sizer_3, 1, wxEXPAND, 0);
-  grid_sizer_2->Add(label_9, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  sizer_4->Add(text_ctrl_8, 0, wxALL, 2);
-  sizer_4->Add(label_10, 0, wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL | wxALL, 2);
-  sizer_4->Add(text_ctrl_9, 0, wxALL, 2);
+  grid_sizer_2->Add(label_9, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  sizer_4->Add(text_ctrl_8, 0, wxALL, 5);
+  sizer_4->Add(label_10, 0, wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
+  sizer_4->Add(text_ctrl_9, 0, wxALL, 5);
   grid_sizer_2->Add(sizer_4, 1, wxEXPAND, 0);
-  grid_sizer_2->Add(label_11, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  grid_sizer_2->Add(combo_box_1, 0, wxALL, 2);
-  grid_sizer_2->Add(label_12, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  sizer_5->Add(combo_box_2, 0, wxALL, 2);
-  sizer_5->Add(check_box_1, 0, wxALL | wxALIGN_CENTER_VERTICAL, 2);
+  grid_sizer_2->Add(label_11, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  grid_sizer_2->Add(combo_box_1, 0, wxALL, 5);
+  grid_sizer_2->Add(label_12, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  sizer_5->Add(combo_box_2, 0, wxALL, 5);
+  sizer_5->Add(check_box_1, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
   grid_sizer_2->Add(sizer_5, 1, wxEXPAND, 0);
-  grid_sizer_2->Add(label_13, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  sizer_6->Add(text_ctrl_10, 0, wxALL | wxALIGN_CENTER_VERTICAL, 2);
-  sizer_6->Add(button_3, 0, wxALL, 2);
+  grid_sizer_2->Add(label_13, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  sizer_6->Add(text_ctrl_10, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
+  sizer_6->Add(button_3, 0, wxALL, 5);
   grid_sizer_2->Add(sizer_6, 1, wxEXPAND, 0);
   grid_sizer_1->Add(grid_sizer_2, 1, wxEXPAND, 0);
   grid_sizer_1->Add(static_line_1, 0, wxEXPAND | wxLEFT | wxRIGHT, 2);
-  sizer_1->Add(button_1, 0, wxLEFT | wxRIGHT, 5);
-  sizer_1->Add(button_2, 0, wxLEFT | wxRIGHT, 5);
-  grid_sizer_1->Add(sizer_1, 1, wxALIGN_RIGHT | wxTOP | wxBOTTOM, 3);
+  sizer_1->Add(button_1, 0, wxALL, 5);
+  sizer_1->Add(button_2, 0, wxALL, 5);
+  grid_sizer_1->Add(sizer_1, 1, wxALIGN_RIGHT, 0);
   SetAutoLayout(true);
   SetSizer(grid_sizer_1);
   grid_sizer_1->Fit(this);
@@ -209,7 +206,7 @@ void Plot3DWiz::SetValue(wxString s)
   }
   else
     text_ctrl_1->SetValue(s);
-  
+
   text_ctrl_1->SetSelection(-1, -1);
 }
 
@@ -374,7 +371,7 @@ wxString Plot3DWiz::GetValue()
   wxString f = combo_box_1->GetValue();
   wxString file = text_ctrl_10->GetValue();
   int xg = text_ctrl_8->GetValue();
-  int yg = text_ctrl_8->GetValue();
+  int yg = text_ctrl_9->GetValue();
   if (pl.Contains(wxT(", ")))
     pl = wxT("[") + pl + wxT("]");
   s += pl;
@@ -421,11 +418,8 @@ wxString Plot3DWiz::GetValue()
   }
   else if (f == _("inline"))
     s = wxT("wx") + s;
-  
-  if (file.Length() == 0 && f == _("inline"))
-    s += wxT(");");
-  else
-    s += wxT(")$");
+
+  s += wxT(")$");
 
   wxConfig::Get()->Write(wxT("Wiz/Plot3D/format"), combo_box_1->GetSelection());
   wxConfig::Get()->Write(wxT("Wiz/Plot3D/pm3d"), check_box_1->GetValue());
@@ -465,7 +459,7 @@ void Plot3DWiz::OnFileBrowse(wxCommandEvent& event)
   wxString file = wxFileSelector(_("Save plot to file"), wxEmptyString,
                                  wxT("plot3d.eps"), wxT("eps"),
                                  _("Postscript file (*.eps)|*.eps|All|*"),
-                                 wxSAVE | wxOVERWRITE_PROMPT);
+                                 wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
   if (file.Length() > 0)
     text_ctrl_10->SetValue(file);
 }
