@@ -1,22 +1,21 @@
-/*
- *  Copyright (C) 2004-2006 Andrej Vodopivec <andrejv@users.sourceforge.net>
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- */
+///
+///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///
+///  This program is free software; you can redistribute it and/or modify
+///  it under the terms of the GNU General Public License as published by
+///  the Free Software Foundation; either version 2 of the License, or
+///  (at your option) any later version.
+///
+///  This program is distributed in the hope that it will be useful,
+///  but WITHOUT ANY WARRANTY; without even the implied warranty of
+///  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+///  GNU General Public License for more details.
+///
+///
+///  You should have received a copy of the GNU General Public License
+///  along with this program; if not, write to the Free Software
+///  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+///
 
 #ifndef PLOT3DWIZ_H
 #define PLOT3DWIZ_H
@@ -28,12 +27,13 @@
 
 #include "BTextCtrl.h"
 
-class Plot3DWiz: public wxDialog {
+class Plot3DWiz: public wxDialog
+{
 public:
   Plot3DWiz(wxWindow* parent, int id, const wxString& title,
-            const wxPoint& pos=wxDefaultPosition,
-            const wxSize& size=wxDefaultSize,
-            long style=wxDEFAULT_DIALOG_STYLE);
+            const wxPoint& pos = wxDefaultPosition,
+            const wxSize& size = wxDefaultSize,
+            long style = wxDEFAULT_DIALOG_STYLE);
   void SetValue(wxString s);
   void Parse(wxString s);
   wxString GetValue();
@@ -44,7 +44,6 @@ private:
   void do_layout();
 protected:
   int type;
-  wxStaticText* label_1;
   wxStaticText* label_2;
   BTextCtrl* text_ctrl_1;
   wxStaticText* label_3;
@@ -77,5 +76,4 @@ protected:
   DECLARE_EVENT_TABLE()
 };
 
-
 #endif // PLOT3DWIZ_H
