@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2006 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -43,6 +43,18 @@ SumCell::~SumCell()
     delete m_next;
 }
 
+void SumCell::SetParent(MathCell *parent, bool all)
+{
+  if (m_base != NULL)
+    m_base->SetParent(parent, true);
+  if (m_under != NULL)
+    m_under->SetParent(parent, true);
+  if (m_over != NULL)
+    m_over->SetParent(parent, true);
+
+  MathCell::SetParent(parent, all);
+}
+
 MathCell* SumCell::Copy(bool all)
 {
   SumCell *tmp = new SumCell;
@@ -101,15 +113,15 @@ void SumCell::RecalculateWidths(CellParser& parser, int fontsize, bool all)
 {
   double scale = parser.GetScale();
 
-  m_signCenter = SCALE_PX(m_signCenter, scale);
-  m_signSize = SCALE_PX(m_signSize, scale);
-  m_signWidth = SCALE_PX(m_signWidth, scale);
+  m_signSize = SCALE_PX(50, scale);
+  m_signWidth = SCALE_PX(30, scale);
+  m_signCenter = SCALE_PX(15, scale);
 
   m_base->RecalculateWidths(parser, fontsize, true);
-  m_under->RecalculateWidths(parser, MAX(8, fontsize - 5), true);
+  m_under->RecalculateWidths(parser, MAX(MC_MIN_SIZE, fontsize - 5), true);
   if (m_over == NULL)
     m_over = new TextCell;
-  m_over->RecalculateWidths(parser, MAX(8, fontsize - 5), true);
+  m_over->RecalculateWidths(parser, MAX(MC_MIN_SIZE, fontsize - 5), true);
 
   m_signCenter = MAX(m_signCenter, m_under->GetFullWidth(scale) / 2);
   m_signCenter = MAX(m_signCenter, m_over->GetFullWidth(scale) / 2);
@@ -122,8 +134,8 @@ void SumCell::RecalculateSize(CellParser& parser, int fontsize, bool all)
 {
   double scale = parser.GetScale();
 
-  m_under->RecalculateSize(parser, MAX(8, fontsize - 5), true);
-  m_over->RecalculateSize(parser, MAX(8, fontsize - 5), true);
+  m_under->RecalculateSize(parser, MAX(MC_MIN_SIZE, fontsize - 5), true);
+  m_over->RecalculateSize(parser, MAX(MC_MIN_SIZE, fontsize - 5), true);
   m_base->RecalculateSize(parser, fontsize, true);
 
   m_center = MAX(m_over->GetMaxHeight() + SCALE_PX(4, scale) + m_signSize / 2,
@@ -145,12 +157,12 @@ void SumCell::Draw(CellParser& parser, wxPoint point, int fontsize, bool all)
     wxPoint base(point), under(point), over(point);
 
     under.x += m_signCenter - m_under->GetFullWidth(scale) / 2;
-    under.y = point.y + m_signSize / 2 + m_under->GetMaxCenter() + 2;
-    m_under->Draw(parser, under, MAX(8, fontsize - 5), true);
+    under.y = point.y + m_signSize / 2 + m_under->GetMaxCenter() + SCALE_PX(2, scale);
+    m_under->Draw(parser, under, MAX(MC_MIN_SIZE, fontsize - 5), true);
 
     over.x += m_signCenter - m_over->GetFullWidth(scale) / 2;
-    over.y = point.y - m_signSize / 2 - m_over->GetMaxDrop() - 2;
-    m_over->Draw(parser, over, MAX(8, fontsize - 5), true);
+    over.y = point.y - m_signSize / 2 - m_over->GetMaxDrop() - SCALE_PX(2, scale);
+    m_over->Draw(parser, over, MAX(MC_MIN_SIZE, fontsize - 5), true);
 
     SetPen(parser);
     if (m_sumStyle == SM_SUM)
@@ -259,6 +271,33 @@ wxString SumCell::ToString(bool all)
   return s;
 }
 
+wxString SumCell::ToTeX(bool all)
+{
+  wxString s;
+  if (m_sumStyle == SM_SUM)
+    s = wxT("\\sum");
+  else
+    s = wxT("\\prod");
+
+
+  s += wxT("_{") + m_under->ToTeX(true) + wxT("}");
+  wxString to = m_over->ToTeX(true);
+  if (to.Length())
+    s += wxT("^{") + to + wxT("}");
+  s += m_base->ToTeX(true);
+
+  s += MathCell::ToTeX(all);
+  return s;
+}
+
+wxString SumCell::ToXML(bool all)
+{
+	return _T("<sm><r>") + m_under->ToXML(true) + _T("</r><r>") +
+			m_over->ToXML(true) + _T("</r><r>") +
+			m_base->ToXML(true) + _T("</r></sm>") +
+			MathCell::ToXML(all);
+}
+
 void SumCell::SelectInner(wxRect& rect, MathCell** first, MathCell** last)
 {
   *first = NULL;
