@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2006 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -22,28 +22,43 @@
 
 struct style
 {
-  style() : bold(false), italic(false), underlined(false)
-  { }
-  ;
-  wxString color;
+  style() :
+    bold(false), italic(false), underlined(false)
+    {  };
+  wxColour color;
+  wxString font;
+  int fontSize;
   bool bold;
   bool italic;
   bool underlined;
 };
 
-enum {
-  TS_NORMAL_TEXT = 0,
-  TS_HIDDEN_GROUP = 1,
-  TS_MAIN_PROMPT = 2,
-  TS_OTHER_PROMPT = 3,
-  TS_LABEL = 4,
-  TS_SPECIAL_CONSTANT = 5,
-  TS_INPUT = 6,
-  TS_NUMBER = 7,
-  TS_STRING = 8,
-  TS_GREEK_CONSTANT = 9,
-  TS_VARIABLES = 10,
-  TS_HIGHLIGHT = 11
+enum
+{
+  TS_DEFAULT = 0,
+  TS_VARIABLE,
+  TS_FUNCTION,
+  TS_NUMBER,
+  TS_GREEK_CONSTANT,
+  TS_SPECIAL_CONSTANT,
+  TS_STRING,
+  TS_MAIN_PROMPT,
+  TS_OTHER_PROMPT,
+  TS_LABEL,
+  TS_INPUT,
+  TS_HIGHLIGHT,
+  TS_TEXT_BACKGROUND,
+  TS_TEXT,
+  TS_SECTION,
+  TS_TITLE,
+  TS_ERROR,
+  TS_CELL_BRACKET,
+  TS_ACTIVE_CELL_BRACKET,
+  TS_CURSOR,
+  TS_SELECTION,
+  TS_OUTDATED
 };
 
+#define STYLE_NUM 22
+
 #endif
