@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2006 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -25,16 +25,30 @@
 
 #include <wx/socket.h>
 #include <wx/config.h>
-#include <wx/dnd.h>
 #include <wx/process.h>
 
+#if defined (__WXMSW__)
+ #include <wx/msw/helpchm.h>
+#else
+ #include <wx/html/helpctrl.h>
+#endif
+
 #define SOCKET_SIZE 1024
+#define DOCUMENT_VERSION_MAJOR 1
+#define DOCUMENT_VERSION_MINOR 0
 
 class MyApp : public wxApp
 {
 public:
   virtual bool OnInit();
   wxLocale m_locale;
+  void NewWindow(wxString file = wxEmptyString);
+#if defined (__WXMAC__)
+  wxWindowList topLevelWindows;
+  void OnFileMenu(wxCommandEvent &ev);
+  virtual void MacNewFile();
+  virtual void MacOpenFile(const wxString& file);
+#endif
 };
 
 DECLARE_APP(MyApp)
@@ -53,13 +67,13 @@ public:
   {
     m_openFile = file;
   }
-  void SendMaxima(wxString s, bool clear = true, bool out = true,
-                  bool silent = true,   // sends input to maxima, also add to command
-                  bool split = true);   // history...
-  bool ReadBatchFile(wxString file);
+  void SendMaxima(wxString s);
+  void OpenFile(wxString file,
+                wxString command = wxEmptyString); // Open a file
 protected:
-  void EnterCommand(wxCommandEvent& event);        // enter in the command line
-
+  void OnRecentDocument(wxCommandEvent& event);
+  void OnIdle(wxIdleEvent& event);
+  void MenuCommand(wxString cmd);                  //
   void FileMenu(wxCommandEvent& event);            //
   void MaximaMenu(wxCommandEvent& event);          //
   void AlgebraMenu(wxCommandEvent& event);         //
@@ -71,11 +85,9 @@ protected:
   void HelpMenu(wxCommandEvent& event);            //
   void EditMenu(wxCommandEvent& event);            //
   void Interrupt(wxCommandEvent& event);           //
-  void OnMonitorFile(wxCommandEvent& event);       //
-  void OnActivate(wxActivateEvent& event);         //
-  void OnSetFocus(wxFocusEvent& event);            //
   void UpdateMenus(wxUpdateUIEvent& event);        //
   void UpdateToolBar(wxUpdateUIEvent& event);      //
+  void UpdateSlider(wxUpdateUIEvent& event);       //
   void OnProcessEvent(wxProcessEvent& event);      //
   void PopupMenu(wxCommandEvent& event);           //
 
@@ -86,16 +98,15 @@ protected:
   void DoConsoleAppend(wxString s, int type,       //
                        bool newLine = true, bool bigSkip = true);
   void DoRawConsoleAppend(wxString s, int type,    //
-                          bool newLine = true);    //
+                          bool newLine = true,     //
+                          bool hide = false);      //
 
   void EditInputMenu(wxCommandEvent& event);       //
-  void ReEvaluate(wxCommandEvent& event);          //
-  void PrependCell(wxCommandEvent& event);         //
-  void HandleCellEvent(wxCommandEvent& event);
-
-  void HandleMainPrompt(wxString prompt);
-
-  wxString SplitInput(wxString input);
+  void EvaluateEvent(wxCommandEvent& event);     //
+  void InsertMenu(wxCommandEvent& event);          //
+  void SliderEvent(wxScrollEvent& event);
+  void DumpProcessOutput();
+  void TryEvaluateNextInQueue();
 
 #if WXM_PRINT
   void CheckForPrintingSupport();
@@ -107,7 +118,7 @@ protected:
   bool StartMaxima();                              // starts maxima (uses getCommand)
   void CleanUp();                                  // shuts down server and client on exit
   void OnClose(wxCloseEvent& event);               // close wxMaxima window
-  wxString GetCommand();                           // returns the command to start maxima
+  wxString GetCommand(bool params = true);         // returns the command to start maxima
                                                    //    (uses guessConfiguration)
 
   void ReadFirstPrompt();            // reads everything before first prompt
@@ -115,8 +126,6 @@ protected:
   void ReadPrompt();                 // reads prompts
   void ReadMath();                   // reads output other than prompts
   void ReadLispError();              // lisp errors (no prompt prefix/suffix)
-  void OpenFile(wxString file,
-                wxString command = wxEmptyString);      // Open a file
 #ifndef __WXMSW__
   void ReadProcessOutput();          // reads output of maxima command
 #endif
@@ -125,7 +134,13 @@ protected:
   void KillMaxima();                 // kills the maxima process
   void ResetTitle(bool saved);
   void FirstOutput(wxString s);
-  wxString RemoveTabs(wxString s);
+
+  // loading functions
+  bool OpenWXMFile(wxString file, MathCtrl *document, bool clearDocument = true);
+  bool OpenWXMXFile(wxString file, MathCtrl *document, bool clearDocument = true);
+  GroupCell* CreateTreeFromXMLNode(wxXmlNode *xmlcells, wxString wxmxfilename = wxEmptyString);
+  GroupCell* CreateTreeFromWXMCode(wxArrayString *wxmLines);
+
   wxSocketBase *m_client;
   wxSocketServer *m_server;
   bool m_isConnected;
@@ -139,16 +154,10 @@ protected:
   wxString m_promptSuffix;
   wxString m_promptPrefix;
   wxString m_firstPrompt;
-  wxString m_commentPrefix;
-  wxString m_newInput;
   bool m_readingPrompt;
   bool m_dispReadOut;               // what is displayed in statusbar
   bool m_inLispMode;                // don't add ; in lisp mode
-  bool m_inPrompt;
-  bool m_inInsertMode;
   wxString m_lastPrompt;
-  wxString m_monitorFile;
-  time_t m_monitorTime;
   wxString m_lastPath;
   MathParser m_MParser;
   wxPrintData* m_printData;
@@ -158,11 +167,14 @@ protected:
   bool m_closing;
   wxString m_openFile;
   wxString m_currentFile;
-  wxArrayString m_batchFileLines;
-  unsigned int m_batchFilePosition;
   bool m_fileSaved;
   bool m_variablesOK;
   wxString m_helpFile;
+#if defined (__WXMSW__)
+  wxCHMHelpController m_helpCtrl;
+#else
+  wxHtmlHelpController m_helpCtrl;
+#endif
   DECLARE_EVENT_TABLE()
 };
 
